@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -51,6 +51,38 @@ RDTweakedNativeStyle::~RDTweakedNativeStyle()
 {
 }
 
+QRect RDTweakedNativeStyle::subControlRect(ComplexControl cc, const QStyleOptionComplex *opt,
+                                           SubControl sc, const QWidget *widget) const
+{
+  if(cc == QStyle::CC_ToolButton)
+  {
+    int indicatorWidth = proxy()->pixelMetric(PM_MenuButtonIndicator, opt, widget);
+
+    QRect ret = opt->rect;
+
+    const QStyleOptionToolButton *toolbutton = qstyleoption_cast<const QStyleOptionToolButton *>(opt);
+
+    // return the normal rect if there's no menu
+    if(!shouldDrawToolButtonMenuArrow(toolbutton))
+    {
+      return ret;
+    }
+
+    if(sc == QStyle::SC_ToolButton)
+    {
+      ret.setRight(ret.right() - indicatorWidth);
+    }
+    else if(sc == QStyle::SC_ToolButtonMenu)
+    {
+      ret.setLeft(ret.right() - indicatorWidth);
+    }
+
+    return ret;
+  }
+
+  return QProxyStyle::subControlRect(cc, opt, sc, widget);
+}
+
 QRect RDTweakedNativeStyle::subElementRect(SubElement element, const QStyleOption *opt,
                                            const QWidget *widget) const
 {
@@ -74,12 +106,11 @@ QSize RDTweakedNativeStyle::sizeFromContents(ContentsType type, const QStyleOpti
 {
   QSize sz = size;
 
-  // Toolbuttons are always at least icon sized, for consistency.
   if(type == QStyle::CT_ToolButton)
   {
     const QStyleOptionToolButton *toolbutton = qstyleoption_cast<const QStyleOptionToolButton *>(opt);
     if(toolbutton)
-      sz = sz.expandedTo(toolbutton->iconSize);
+      sz = adjustToolButtonSize(toolbutton, sz, widget);
   }
 
   // menu bar items can be sized for both the icon *and* the text
@@ -152,13 +183,24 @@ void RDTweakedNativeStyle::drawComplexControl(ComplexControl control, const QSty
     backCol.setAlphaF(0.2);
 
     QStyleOptionToolButton menu;
+    bool hasMenu = false;
+    bool hasSeparateMenu = false;
 
     // draw the menu arrow, if there is one
-    if((toolbutton->subControls & SC_ToolButtonMenu) ||
-       (toolbutton->features & QStyleOptionToolButton::MenuButtonPopup))
+    if(shouldDrawToolButtonMenuArrow(toolbutton))
     {
       menu = *toolbutton;
       menu.rect = subControlRect(control, opt, SC_ToolButtonMenu, widget);
+      hasMenu = true;
+      // We always draw an arrow if a menu is present (normally Qt only does it for MenuButtonPopup,
+      // where there is both a button with a default action and a menu triggered by a small arrow,
+      // and not InstantPopup where there is only a button). If the button uses MenuButtonPopup,
+      // we want to draw a line to distinguish the menu part of the button and the main part,
+      // but we don't need that line if the arrow is decorative only.
+      if(toolbutton->features & QStyleOptionToolButton::MenuButtonPopup)
+      {
+        hasSeparateMenu = true;
+      }
     }
 
     QStyle::State masked = opt->state & (State_On | State_MouseOver);
@@ -171,7 +213,7 @@ void RDTweakedNativeStyle::drawComplexControl(ComplexControl control, const QSty
       QRect rect = opt->rect.adjusted(0, 0, -1, -1);
       p->setPen(opt->palette.color(QPalette::Shadow));
       p->drawRect(rect);
-      if(menu.rect.isValid())
+      if(hasSeparateMenu)
         p->drawLine(menu.rect.topLeft(), menu.rect.bottomLeft());
 
       // when the mouse is over, make it a little stronger
@@ -189,7 +231,7 @@ void RDTweakedNativeStyle::drawComplexControl(ComplexControl control, const QSty
     // draw the label text/icon
     drawControl(CE_ToolButtonLabel, &labelTextIcon, p, widget);
 
-    if(menu.rect.isValid())
+    if(hasMenu)
     {
       menu.rect.adjust(2, 0, 0, 0);
       drawPrimitive(PE_IndicatorArrowDown, &menu, p, widget);
@@ -552,3 +594,34 @@ void RDTweakedNativeStyle::drawControl(ControlElement control, const QStyleOptio
 
   QProxyStyle::drawControl(control, opt, p, widget);
 }
+
+bool RDTweakedNativeStyle::shouldDrawToolButtonMenuArrow(const QStyleOptionToolButton *toolbutton) const
+{
+  // Qt normally only draws the arrow for MenuButtonPopup; we want it for all tools button with
+  // menus (including InstantPopup).
+  return (toolbutton->subControls & SC_ToolButtonMenu) ||
+         (toolbutton->features & QStyleOptionToolButton::HasMenu);
+}
+
+QSize RDTweakedNativeStyle::adjustToolButtonSize(const QStyleOptionToolButton *toolbutton,
+                                                 const QSize &size, const QWidget *widget) const
+{
+  QSize sz = size;
+
+  // Toolbuttons are always at least icon sized, for consistency.
+  sz = sz.expandedTo(toolbutton->iconSize);
+
+  if(shouldDrawToolButtonMenuArrow(toolbutton))
+  {
+    // QToolButton::sizeHint automatically increases the width for MenuButtonPopup separate from
+    // calling sizeFromContents. But we want to draw the arrow for all tool buttons with menus,
+    // not just those using MenuButtonPopup. Check for MenuButtonPopup to avoid increasing the
+    // size twice.
+    if(!(toolbutton->features & QStyleOptionToolButton::MenuButtonPopup))
+    {
+      sz.setWidth(sz.width() + proxy()->pixelMetric(PM_MenuButtonIndicator, toolbutton, widget));
+    }
+  }
+
+  return sz;
+}
