@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,6 +27,8 @@
 #include <QPalette>
 #include <QProxyStyle>
 
+class QStyleOptionToolButton;
+
 class RDTweakedNativeStyle : public QProxyStyle
 {
 private:
@@ -35,6 +37,8 @@ public:
   RDTweakedNativeStyle(QStyle *parent);
   ~RDTweakedNativeStyle();
 
+  virtual QRect subControlRect(ComplexControl cc, const QStyleOptionComplex *opt, SubControl sc,
+                               const QWidget *widget = Q_NULLPTR) const override;
   virtual QRect subElementRect(SubElement element, const QStyleOption *option,
                                const QWidget *widget) const override;
   virtual QSize sizeFromContents(ContentsType type, const QStyleOption *option, const QSize &size,
@@ -54,4 +58,7 @@ public:
                            const QWidget *widget = NULL) const override;
 
 protected:
+  bool shouldDrawToolButtonMenuArrow(const QStyleOptionToolButton *toolbutton) const;
+  QSize adjustToolButtonSize(const QStyleOptionToolButton *toolbutton, const QSize &size,
+                             const QWidget *widget) const;
 };
