@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2021 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -155,7 +155,14 @@ public:
     if(!index.isValid())
       return 0;
 
-    return QAbstractItemModel::flags(index) | Qt::ItemIsUserCheckable;
+    RDTreeWidgetItem *item = itemForIndex(index);
+
+    Qt::ItemFlags ret = QAbstractItemModel::flags(index) | Qt::ItemIsUserCheckable;
+
+    if(item->editable(index.column()))
+      ret |= Qt::ItemIsEditable;
+
+    return ret;
   }
 
   QVariant headerData(int section, Qt::Orientation orientation, int role) const override
@@ -231,7 +238,7 @@ public:
 
     bool ret = false;
 
-    if(role == Qt::DisplayRole)
+    if(role == Qt::DisplayRole || role == Qt::EditRole)
     {
       if(index.column() < item->m_text.count())
       {
@@ -371,7 +378,7 @@ RDTreeWidgetItem::~RDTreeWidgetItem()
 
 QVariant RDTreeWidgetItem::data(int column, int role) const
 {
-  if(role == Qt::DisplayRole)
+  if(role == Qt::DisplayRole || role == Qt::EditRole)
   {
     return m_text[column];
   }
@@ -556,6 +563,7 @@ void RDTreeWidgetItem::clear()
   for(RDTreeWidgetItem *c : children)
   {
     c->m_parent = NULL;
+    c->m_widget = NULL;
     delete c;
   }
 
@@ -766,9 +774,14 @@ void RDTreeWidget::collapseAllItems(RDTreeWidgetItem *item)
   collapseAll(m_model->indexForItem(item, 0));
 }
 
-void RDTreeWidget::scrollToItem(RDTreeWidgetItem *node)
+void RDTreeWidget::scrollToItem(RDTreeWidgetItem *item)
 {
-  scrollTo(m_model->indexForItem(node, 0));
+  scrollTo(m_model->indexForItem(item, 0));
+}
+
+void RDTreeWidget::editItem(RDTreeWidgetItem *item)
+{
+  edit(m_model->indexForItem(item, 0));
 }
 
 void RDTreeWidget::clear()
