@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -130,7 +130,7 @@ void EnvironmentEditor::on_variables_currentItemChanged(RDTreeWidgetItem *curren
 
   EnvironmentModification mod = current->tag().value<EnvironmentModification>();
 
-  if(!mod.value.empty())
+  if(!mod.name.empty())
   {
     ui->name->setText(mod.name);
     ui->value->setText(mod.value);
