@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -181,8 +181,9 @@ int GetCurrentPID(const rdcstr &deviceID, const rdcstr &processName)
     RDCLOG("Getting PID from device %s for process '%s'", deviceID.c_str(), processName.c_str());
   }
 
-  // try 5 times, 200ms apart to find the pid
-  for(int i = 0; i < 5; i++)
+  // try 15 times, 200ms apart to find the pid, for a total of 3 seconds timeout waiting for the
+  // process to even be alive
+  for(int i = 0; i < 15; i++)
   {
     Process::ProcessResult pidOutput =
         adbExecCommand(deviceID, StringFormat::Fmt("shell ps -A | grep %s", processName.c_str()));
@@ -323,6 +324,25 @@ bool CheckAndroidServerVersion(const rdcstr &deviceID, ABI abi)
   return false;
 }
 
+Process::ProcessResult ListPackages(const rdcstr &deviceID, const rdcstr &parameters)
+{
+  Process::ProcessResult result = adbExecCommand(deviceID, "shell getprop ro.build.version.sdk");
+
+  int sdkVersion = -1;
+  if(result.retCode == 0)
+    sdkVersion = atoi(result.strStdout.c_str());
+
+  // Android 13 and higher
+  if(sdkVersion >= 33)
+    result = adbExecCommand(deviceID,
+                            "shell pm list packages --user $(am get-current-user) " + parameters);
+
+  if(result.retCode != 0 || sdkVersion < 33)
+    result = adbExecCommand(deviceID, "shell pm list packages " + parameters);
+
+  return result;
+}
+
 RDResult InstallRenderDocServer(const rdcstr &deviceID)
 {
   ResultCode result = ResultCode::Succeeded;
@@ -464,8 +484,7 @@ RDResult InstallRenderDocServer(const rdcstr &deviceID)
   }
 
   // Ensure installation succeeded. We should have as many lines as abis we installed
-  Process::ProcessResult adbCheck =
-      adbExecCommand(deviceID, "shell pm list packages " RENDERDOC_ANDROID_PACKAGE_BASE);
+  Process::ProcessResult adbCheck = ListPackages(deviceID, RENDERDOC_ANDROID_PACKAGE_BASE);
 
   if(adbCheck.strStdout.empty())
   {
@@ -498,7 +517,7 @@ bool RemoveRenderDocAndroidServer(const rdcstr &deviceID)
     adbExecCommand(deviceID, "uninstall " + packageName);
 
     // Ensure uninstall succeeded
-    rdcstr adbCheck = adbExecCommand(deviceID, "shell pm list packages " + packageName).strStdout;
+    rdcstr adbCheck = ListPackages(deviceID, packageName).strStdout;
 
     if(!adbCheck.empty())
     {
@@ -601,7 +620,7 @@ struct AndroidRemoteServer : public RemoteServer
     {
       SCOPED_TIMER("Fetching android packages and activities");
 
-      rdcstr adbStdout = Android::adbExecCommand(m_deviceID, "shell pm list packages -3").strStdout;
+      rdcstr adbStdout = Android::ListPackages(m_deviceID, "-3").strStdout;
 
       rdcarray<rdcstr> lines;
       split(adbStdout, lines, '\n');
@@ -629,7 +648,7 @@ struct AndroidRemoteServer : public RemoteServer
       }
 
       // also fetch the system packages but mark them as hidden folders
-      adbStdout = Android::adbExecCommand(m_deviceID, "shell pm list packages -s").strStdout;
+      adbStdout = Android::ListPackages(m_deviceID, "-s").strStdout;
 
       split(adbStdout, lines, '\n');
       for(rdcstr &line : lines)
@@ -875,10 +894,14 @@ struct AndroidController : public IDeviceProtocolHandler
 
       cmd->meth();
 
-      Atomic::Inc32(&cmd->done);
-
       if(cmd->selfdelete)
+      {
         delete cmd;
+      }
+      else
+      {
+        Atomic::Inc32(&cmd->done);
+      }
     }
   }
 
@@ -1003,9 +1026,7 @@ struct AndroidController : public IDeviceProtocolHandler
       }
 
       rdcstr packagesOutput =
-          Android::adbExecCommand(deviceID,
-                                  "shell pm list packages " RENDERDOC_ANDROID_PACKAGE_BASE)
-              .strStdout.trimmed();
+          Android::ListPackages(deviceID, RENDERDOC_ANDROID_PACKAGE_BASE).strStdout.trimmed();
 
       rdcarray<rdcstr> packages;
       split(packagesOutput, packages, '\n');
@@ -1078,7 +1099,7 @@ struct AndroidController : public IDeviceProtocolHandler
     });
 
     // allow the package to start and begin listening before we return
-    Threading::Sleep(1500);
+    Threading::Sleep(8000);
 
     return result;
   }
