@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -47,7 +47,9 @@ struct BindingElement
            filter == o.filter && addressU == o.addressU && addressV == o.addressV &&
            addressW == o.addressW && mipBias == o.mipBias && maxAnisotropy == o.maxAnisotropy &&
            compareFunction == o.compareFunction && minLOD == o.minLOD && maxLOD == o.maxLOD &&
-           borderColor == o.borderColor && unnormalized == o.unnormalized;
+           borderColorValue.uintValue == o.borderColorValue.uintValue &&
+           borderColorType == o.borderColorType && unnormalized == o.unnormalized &&
+           srgbBorder == o.srgbBorder && seamless == o.seamless;
   }
   bool operator<(const BindingElement &o) const
   {
@@ -99,10 +101,16 @@ struct BindingElement
       return minLOD < o.minLOD;
     if(!(maxLOD == o.maxLOD))
       return maxLOD < o.maxLOD;
-    if(!(borderColor == o.borderColor))
-      return borderColor < o.borderColor;
+    if(!(borderColorValue.uintValue == o.borderColorValue.uintValue))
+      return borderColorValue.uintValue < o.borderColorValue.uintValue;
+    if(!(borderColorType == o.borderColorType))
+      return borderColorType < o.borderColorType;
     if(!(unnormalized == o.unnormalized))
       return unnormalized < o.unnormalized;
+    if(!(srgbBorder == o.srgbBorder))
+      return srgbBorder < o.srgbBorder;
+    if(!(seamless == o.seamless))
+      return seamless < o.seamless;
     return false;
   }
 
@@ -174,21 +182,41 @@ set's inline block data.
   float maxAnisotropy = 0.0f;
   DOCUMENT("For samplers - the :class:`CompareFunction` for comparison samplers.");
   CompareFunction compareFunction = CompareFunction::AlwaysTrue;
-  DOCUMENT("For samplers - the minimum mip level that can be used.");
+  DOCUMENT("For samplers and image views - the minimum mip level that can be used.");
   float minLOD = 0.0f;
   DOCUMENT("For samplers - the maximum mip level that can be used.");
   float maxLOD = 0.0f;
-  DOCUMENT(R"(For samplers - the RGBA border color.
+  DOCUMENT(R"(For samplers - the RGBA border color value. Typically the float tuple inside will be used,
+but the exact component type can be checked with :data:`borderColorType`.
 
-:type: Tuple[float,float,float,float]
+:type: PixelValue
 )");
-  rdcfixedarray<float, 4> borderColor = {0.0f, 0.0f, 0.0f, 0.0f};
+  PixelValue borderColorValue = {};
+  DOCUMENT(R"(For samplers - the RGBA border color type. This determines how the data in
+:data:`borderColorValue` will be interpreted.
+
+:type: CompType
+)");
+  CompType borderColorType = CompType::Float;
+  DOCUMENT(R"(The swizzle applied to samplers. Primarily for ycbcr samplers applied before
+conversion but for non-ycbcr samplers can be used for implementations that require sampler swizzle
+information for border colors.
+
+:type: TextureSwizzle4
+)");
+  TextureSwizzle4 samplerSwizzle;
+  DOCUMENT(
+      "For samplers - ``True`` if the border colour is swizzled with an sRGB formatted image.");
+  bool srgbBorder = false;
   DOCUMENT("For samplers - ``True`` if unnormalized co-ordinates are used in this sampler.");
   bool unnormalized = false;
 
   DOCUMENT("``True`` if this is an inline uniform block binding.");
   bool inlineBlock = false;
 
+  DOCUMENT("``True`` if this sampler is seamless across cubemap boundaries (the default).");
+  bool seamless = true;
+
   DOCUMENT(R"(For samplers - the :class:`ResourceId` of the ycbcr conversion object associated with
 this sampler.
 )");
@@ -198,11 +226,6 @@ this sampler.
   YcbcrConversion ycbcrModel;
   DOCUMENT("For ycbcr samplers - the :class:`YcbcrRange` used for conversion.");
   YcbcrRange ycbcrRange;
-  DOCUMENT(R"(For ycbcr samplers - The swizzle applied before conversion.
-
-:type: TextureSwizzle4
-)");
-  TextureSwizzle4 ycbcrSwizzle;
   DOCUMENT("For ycbcr samplers - the :class:`ChromaSampleLocation` X-axis chroma offset.");
   ChromaSampleLocation xChromaOffset;
   DOCUMENT("For ycbcr samplers - the :class:`ChromaSampleLocation` Y-axis chroma offset.");
