@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -641,6 +641,7 @@ DXGI_FORMAT ResourceFormat2DXGIFormat(ResourceFormat format)
       {
         case CompType::UInt: return DXGI_FORMAT_R32_UINT;
         case CompType::SInt: return DXGI_FORMAT_R32_SINT;
+        case CompType::Depth: return DXGI_FORMAT_D32_FLOAT;
         default: return DXGI_FORMAT_R32_FLOAT;
       }
     }
@@ -810,8 +811,7 @@ RDResult write_dds_to_file(FILE *f, const write_dds_data &data)
   // special case a couple of formats to write out non-DX10 style, for
   // backwards compatibility
   if(data.format.compByteWidth * data.format.compCount <= 4 &&
-     data.format.type == ResourceFormatType::Regular &&
-     (data.format.compType == CompType::UNorm || data.format.compType == CompType::UNormSRGB))
+     data.format.type == ResourceFormatType::Regular && data.format.compType == CompType::UNorm)
   {
     uint32_t bits = data.format.compByteWidth * 8;
 
@@ -856,17 +856,17 @@ RDResult write_dds_to_file(FILE *f, const write_dds_data &data)
     header.ddspf.dwBBitMask = 0x001f;
     header.ddspf.dwABitMask = 0x0000;
   }
-  else if(data.format.type == ResourceFormatType::BC1)
+  else if(data.format.type == ResourceFormatType::BC1 && data.format.compType == CompType::UNorm)
   {
     header.ddspf.dwFlags = DDPF_FOURCC;
     header.ddspf.dwFourCC = MAKE_FOURCC('D', 'X', 'T', '1');
   }
-  else if(data.format.type == ResourceFormatType::BC2)
+  else if(data.format.type == ResourceFormatType::BC2 && data.format.compType == CompType::UNorm)
   {
     header.ddspf.dwFlags = DDPF_FOURCC;
     header.ddspf.dwFourCC = MAKE_FOURCC('D', 'X', 'T', '3');
   }
-  else if(data.format.type == ResourceFormatType::BC3)
+  else if(data.format.type == ResourceFormatType::BC3 && data.format.compType == CompType::UNorm)
   {
     header.ddspf.dwFlags = DDPF_FOURCC;
     header.ddspf.dwFourCC = MAKE_FOURCC('D', 'X', 'T', '5');
@@ -1134,6 +1134,14 @@ RDResult load_dds_from_file(StreamReader *reader, read_dds_data &ret)
         if(bits[i] < bits[0] && bits[i] > 0)
           bitWidth = bits[i];
 
+      if(bitWidth == 0)
+      {
+        RETURN_ERROR_RESULT(ResultCode::ImageUnsupported,
+                            "Unsupported RGBA mask: %08x %08x %08x %08x", header.ddspf.dwRBitMask,
+                            header.ddspf.dwGBitMask, header.ddspf.dwBBitMask,
+                            header.ddspf.dwABitMask);
+      }
+
       ret.format.compByteWidth = uint8_t(bitWidth / 8);
       ret.format.compCount = uint8_t(header.ddspf.dwRGBBitCount / bitWidth);
       ret.format.compType = CompType::UNorm;
