@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -259,6 +259,10 @@ cbuffer DebugSampleOperation REG(b0)
 #define FLT_EPSILON 1.192092896e-07f
 #endif
 
+// we pick a space that hopefully no-one else will use
+// must match the define in quadoverdraw.hlsl
+#define QUADOVERDRAW_UAV_SPACE 105202922
+
 // histogram/minmax is calculated in blocks of NxN each with MxM tiles.
 // e.g. a tile is 32x32 pixels, then this is arranged in blocks of 32x32 tiles.
 // 1 compute thread = 1 tile, 1 compute group = 1 block
