@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -38,6 +38,7 @@ class WrappedID3D11DeviceContext;
 
 class AMDCounters;
 class NVCounters;
+class NVD3D11Counters;
 class IntelCounters;
 struct D3D11CounterContext;
 
@@ -359,6 +360,7 @@ private:
 
   AMDCounters *m_pAMDCounters = NULL;
   NVCounters *m_pNVCounters = NULL;
+  NVD3D11Counters *m_pNVPerfCounters = NULL;
   IntelCounters *m_pIntelCounters = NULL;
 
   WrappedID3D11Device *m_pDevice = NULL;
