@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -206,7 +206,7 @@ struct BakedCmdListInfo
   UINT nodeMask;
   D3D12RenderState state;
 
-  rdcarray<D3D12_RESOURCE_BARRIER> barriers;
+  BarrierSet barriers;
 
   ResourceId parentList;
 
@@ -386,6 +386,12 @@ struct D3D12CommandData
   void AddAction(const ActionDescription &a);
   void AddEvent();
   void AddUsage(const D3D12RenderState &state, D3D12ActionTreeNode &actionNode);
-  void AddUsage(D3D12ActionTreeNode &actionNode, ResourceId id, uint32_t EID, ResourceUsage usage);
-  void AddUsage(ResourceId id, ResourceUsage usage);
+
+  void AddUsageForBindInRootSig(const D3D12RenderState &state, D3D12ActionTreeNode &actionNode,
+                                const D3D12RenderState::RootSignature *rootsig,
+                                D3D12_DESCRIPTOR_RANGE_TYPE type, const Bindpoint &b);
+
+  void AddResourceUsage(D3D12ActionTreeNode &actionNode, ResourceId id, uint32_t EID,
+                        ResourceUsage usage);
+  void AddCPUUsage(ResourceId id, ResourceUsage usage);
 };
