@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -35,7 +35,7 @@
 
 // we need to use the most-derived native interface all over the place. To make things easier when
 // new versions come out we typedef it here when we don't need the specific interface
-using ID3D12GraphicsCommandListX = ID3D12GraphicsCommandList6;
+using ID3D12GraphicsCommandListX = ID3D12GraphicsCommandList9;
 
 // replay only class for handling marker regions
 struct D3D12MarkerRegion
@@ -125,6 +125,92 @@ BlendMultiplier MakeBlendMultiplier(D3D12_BLEND blend, bool alpha);
 BlendOperation MakeBlendOp(D3D12_BLEND_OP op);
 StencilOperation MakeStencilOp(D3D12_STENCIL_OP op);
 
+// wrapper around D3D12_RESOURCE_STATES and D3D12_BARRIER_LAYOUT to handle resources that could be
+// in either and varying support
+struct D3D12ResourceLayout
+{
+  D3D12ResourceLayout() : value(0)
+  {
+    RDCCOMPILE_ASSERT(sizeof(*this) == sizeof(D3D12_BARRIER_LAYOUT),
+                      "Layout/state wrapper is wrongly sized");
+    RDCCOMPILE_ASSERT(sizeof(*this) == sizeof(D3D12_RESOURCE_STATES),
+                      "Layout/state wrapper is wrongly sized");
+  }
+  static D3D12ResourceLayout FromStates(D3D12_RESOURCE_STATES s)
+  {
+    return D3D12ResourceLayout((uint32_t)s);
+  }
+  static D3D12ResourceLayout FromLayout(D3D12_BARRIER_LAYOUT s)
+  {
+    return D3D12ResourceLayout(uint32_t(s) | LayoutBit);
+  }
+  bool IsLayout() const { return (value & LayoutBit) != 0; }
+  bool IsStates() const { return (value & LayoutBit) == 0; }
+  D3D12_RESOURCE_STATES ToStates() const { return D3D12_RESOURCE_STATES(value & ~LayoutBit); }
+  D3D12_BARRIER_LAYOUT ToLayout() const
+  {
+    if(value == D3D12_BARRIER_LAYOUT_UNDEFINED)
+      return D3D12_BARRIER_LAYOUT_UNDEFINED;
+    return D3D12_BARRIER_LAYOUT(value & ~LayoutBit);
+  }
+  bool operator==(const D3D12ResourceLayout &o) const { return value == o.value; }
+  bool operator!=(const D3D12ResourceLayout &o) const { return !(*this == o); }
+private:
+  explicit D3D12ResourceLayout(uint32_t v) : value(v) {}
+  // layouts are an enum so this bit should hopefully never be used. Note that LAYOUT_UNDEFINED is
+  // ~0U and annoyingly it has to be specified for buffers (instead of D3D12_BARRIER_LAYOUT_COMMON)
+  // so we need to special-case it.
+  // states are a bitmask but they only use just over 6 hex digits so far, and we assume they won't
+  // be extended (or not by much).
+  // We set the bit for layouts and not for states so that we can serialise this
+  // backwards-compatibly and replace any serialised D3D12_RESOURCE_STATES and have them appear as
+  // the correct enum.
+  static constexpr uint32_t LayoutBit = 0x80000000U;
+  uint32_t value;
+};
+
+typedef rdcarray<D3D12ResourceLayout> SubresourceStateVector;
+
+struct BarrierSet
+{
+  enum AccessType
+  {
+    SRVAccess,
+    CopySourceAccess,
+    CopyDestAccess,
+    ResolveSourceAccess,
+  };
+
+  // set up the barriers to barrier this resource a given type of access
+  void Configure(ID3D12Resource *res, const SubresourceStateVector &states, AccessType access);
+
+  // apply the barrier set onto this list
+  void Apply(ID3D12GraphicsCommandListX *list);
+
+  // unapply the barrier set - used for when we are barrier'ing a resource, doing some work, then
+  // barrier'ing it back to the original state
+  void Unapply(ID3D12GraphicsCommandListX *list);
+
+  void clear()
+  {
+    barriers.clear();
+    newBarriers.clear();
+    newToOldBarriers.clear();
+  }
+  bool empty() const { return barriers.empty() && newBarriers.empty(); }
+  void swap(BarrierSet &other)
+  {
+    barriers.swap(other.barriers);
+    newBarriers.swap(other.newBarriers);
+  }
+
+  rdcarray<D3D12_RESOURCE_BARRIER> barriers;
+  rdcarray<D3D12_TEXTURE_BARRIER> newBarriers;
+  // these are used specifically when we need to record a transition from a new layout to an old
+  // state. Because we execute barriers before newBarriers we can do the other way without this
+  rdcarray<D3D12_RESOURCE_BARRIER> newToOldBarriers;
+};
+
 // similar to RDCUNIMPLEMENTED but for things that are hit often so we don't want to fire the
 // debugbreak.
 #define D3D12NOTIMP(...)                                \
@@ -137,17 +223,22 @@ StencilOperation MakeStencilOp(D3D12_STENCIL_OP op);
     }                                                   \
   }
 
-// uncomment this to cause every internal ExecuteCommandLists to immediately call
-// FlushLists(), and to only submit one command list at once to narrow
-// down the cause of device lost errors
-#define SINGLE_FLUSH_VALIDATE OPTION_OFF
-
 // uncomment this to get verbose debugging about when/where/why partial command
 // buffer replay is happening
 #define VERBOSE_PARTIAL_REPLAY OPTION_OFF
 
+D3D12_DEPTH_STENCIL_DESC2 Upconvert(const D3D12_DEPTH_STENCIL_DESC1 &desc);
+D3D12_RASTERIZER_DESC2 Upconvert(const D3D12_RASTERIZER_DESC &desc);
+
 ShaderStageMask ConvertVisibility(D3D12_SHADER_VISIBILITY ShaderVisibility);
 UINT GetNumSubresources(ID3D12Device *dev, const D3D12_RESOURCE_DESC *desc);
+inline UINT GetNumSubresources(ID3D12Device *dev, const D3D12_RESOURCE_DESC1 *desc)
+{
+  // D3D12_RESOURCE_DESC is the same as the start of D3D12_RESOURCE_DESC1
+  D3D12_RESOURCE_DESC desc0;
+  memcpy(&desc0, desc, sizeof(desc0));
+  return GetNumSubresources(dev, &desc0);
+}
 
 class WrappedID3D12Device;
 
@@ -358,7 +449,7 @@ struct D3D12RootSignature
 
   D3D12_ROOT_SIGNATURE_FLAGS Flags = D3D12_ROOT_SIGNATURE_FLAG_NONE;
   rdcarray<D3D12RootSignatureParameter> Parameters;
-  rdcarray<D3D12_STATIC_SAMPLER_DESC> StaticSamplers;
+  rdcarray<D3D12_STATIC_SAMPLER_DESC1> StaticSamplers;
 };
 
 DECLARE_REFLECTION_STRUCT(D3D12RootSignature);
@@ -413,6 +504,9 @@ struct D3D12CommandSignature
   SERIALISE_INTERFACE(ID3D12GraphicsCommandList4); \
   SERIALISE_INTERFACE(ID3D12GraphicsCommandList5); \
   SERIALISE_INTERFACE(ID3D12GraphicsCommandList6); \
+  SERIALISE_INTERFACE(ID3D12GraphicsCommandList7); \
+  SERIALISE_INTERFACE(ID3D12GraphicsCommandList8); \
+  SERIALISE_INTERFACE(ID3D12GraphicsCommandList9); \
   SERIALISE_INTERFACE(ID3D12RootSignature);        \
   SERIALISE_INTERFACE(ID3D12Resource);             \
   SERIALISE_INTERFACE(ID3D12QueryHeap);            \
@@ -473,8 +567,8 @@ struct D3D12_EXPANDED_PIPELINE_STATE_STREAM_DESC
   D3D12_STREAM_OUTPUT_DESC StreamOutput = {};
   D3D12_BLEND_DESC BlendState = {};
   UINT SampleMask = 0;
-  D3D12_RASTERIZER_DESC RasterizerState = {};
-  D3D12_DEPTH_STENCIL_DESC1 DepthStencilState = {};
+  D3D12_RASTERIZER_DESC2 RasterizerState = {};
+  D3D12_DEPTH_STENCIL_DESC2 DepthStencilState = {};
   D3D12_INPUT_LAYOUT_DESC InputLayout = {};
   D3D12_INDEX_BUFFER_STRIP_CUT_VALUE IBStripCutValue;
   D3D12_PRIMITIVE_TOPOLOGY_TYPE PrimitiveTopologyType;
@@ -527,7 +621,8 @@ public:
     else
     {
       m_StreamDesc.pPipelineStateSubobjectStream = &m_GraphicsStreamData;
-      m_StreamDesc.SizeInBytes = sizeof(m_GraphicsStreamData);
+      m_StreamDesc.SizeInBytes =
+          offsetof(GraphicsStreamData, VariableVersionedData) + m_VariableVersionedDataLength;
       return &m_StreamDesc;
     }
   }
@@ -543,7 +638,7 @@ public:
   }
 
 private:
-  struct
+  struct GraphicsStreamData
   {
     // graphics properties
     SUBOBJECT_HEADER(ROOT_SIGNATURE);
@@ -566,8 +661,6 @@ private:
     D3D12_CACHED_PIPELINE_STATE CachedPSO = {};
     SUBOBJECT_HEADER(VIEW_INSTANCING);
     D3D12_VIEW_INSTANCING_DESC ViewInstancing = {};
-    SUBOBJECT_HEADER(RASTERIZER);
-    D3D12_RASTERIZER_DESC RasterizerState = {};
     SUBOBJECT_HEADER(RENDER_TARGET_FORMATS);
     D3D12_RT_FORMAT_ARRAY RTVFormats = {};
     SUBOBJECT_HEADER(DEPTH_STENCIL_FORMAT);
@@ -582,20 +675,27 @@ private:
     UINT SampleMask = 0;
     SUBOBJECT_HEADER(FLAGS);
     D3D12_PIPELINE_STATE_FLAGS Flags = D3D12_PIPELINE_STATE_FLAG_NONE;
-    SUBOBJECT_HEADER(DEPTH_STENCIL1);
-    D3D12_DEPTH_STENCIL_DESC1 DepthStencilState = {};
-#if ENABLED(RDOC_X64)
-    UINT pad0;
-#endif
     SUBOBJECT_HEADER(BLEND);
     D3D12_BLEND_DESC BlendState = {};
 #if ENABLED(RDOC_X64)
-    UINT pad1;
+    UINT pad0;
 #endif
     SUBOBJECT_HEADER(SAMPLE_DESC);
     DXGI_SAMPLE_DESC SampleDesc = {};
+#if ENABLED(RDOC_X64)
+    UINT pad1;
+#endif
+    // since data must be tightly packed, the final structs that are versioned and may not be
+    // supported are pushed in here
+    byte alignas(void *) VariableVersionedData[
+        // depth-stencil is versioned for separate stencil masks
+        sizeof(D3D12_DEPTH_STENCIL_DESC2) + sizeof(void *) +
+        // rasterization is versioned for line raster mode
+        sizeof(D3D12_RASTERIZER_DESC2) + sizeof(void *)];
   } m_GraphicsStreamData;
 
+  size_t m_VariableVersionedDataLength;
+
   struct
   {
     // compute properties
@@ -663,6 +763,7 @@ DECLARE_REFLECTION_ENUM(D3D12_COLOR_WRITE_ENABLE);
 DECLARE_REFLECTION_ENUM(D3D12_DEPTH_WRITE_MASK);
 DECLARE_REFLECTION_ENUM(D3D12_VIEW_INSTANCING_FLAGS);
 DECLARE_REFLECTION_ENUM(D3D12_RESOLVE_MODE);
+DECLARE_REFLECTION_ENUM(D3D12_LINE_RASTERIZATION_MODE);
 DECLARE_REFLECTION_ENUM(D3D12_WRITEBUFFERIMMEDIATE_MODE);
 DECLARE_REFLECTION_ENUM(D3D12_COMMAND_LIST_FLAGS);
 DECLARE_REFLECTION_ENUM(D3D12_RENDER_PASS_FLAGS);
@@ -673,6 +774,7 @@ DECLARE_REFLECTION_ENUM(D3D12_SHADING_RATE_COMBINER);
 DECLARE_REFLECTION_ENUM(D3D12_ROOT_SIGNATURE_FLAGS);
 DECLARE_REFLECTION_ENUM(D3D12_ROOT_PARAMETER_TYPE);
 DECLARE_REFLECTION_ENUM(D3D12_ROOT_DESCRIPTOR_FLAGS);
+DECLARE_REFLECTION_ENUM(D3D12_SAMPLER_FLAGS);
 DECLARE_REFLECTION_ENUM(D3D12_SHADER_VISIBILITY);
 DECLARE_REFLECTION_ENUM(D3D12_STATIC_BORDER_COLOR);
 DECLARE_REFLECTION_ENUM(D3D12_DESCRIPTOR_RANGE_TYPE);
@@ -680,6 +782,11 @@ DECLARE_REFLECTION_ENUM(D3D12_DESCRIPTOR_RANGE_FLAGS);
 DECLARE_REFLECTION_ENUM(D3D12_TILE_COPY_FLAGS);
 DECLARE_REFLECTION_ENUM(D3D12_TILE_RANGE_FLAGS);
 DECLARE_REFLECTION_ENUM(D3D12_TILE_MAPPING_FLAGS);
+DECLARE_REFLECTION_ENUM(D3D12_BARRIER_ACCESS);
+DECLARE_REFLECTION_ENUM(D3D12_BARRIER_SYNC);
+DECLARE_REFLECTION_ENUM(D3D12_BARRIER_LAYOUT);
+DECLARE_REFLECTION_ENUM(D3D12_BARRIER_TYPE);
+DECLARE_REFLECTION_ENUM(D3D12_TEXTURE_BARRIER_FLAGS);
 
 DECLARE_REFLECTION_STRUCT(D3D12_RESOURCE_DESC);
 DECLARE_REFLECTION_STRUCT(D3D12_COMMAND_QUEUE_DESC);
@@ -687,8 +794,13 @@ DECLARE_REFLECTION_STRUCT(D3D12_SHADER_BYTECODE);
 DECLARE_REFLECTION_STRUCT(D3D12_SO_DECLARATION_ENTRY);
 DECLARE_REFLECTION_STRUCT(D3D12_STREAM_OUTPUT_DESC);
 DECLARE_REFLECTION_STRUCT(D3D12_RASTERIZER_DESC);
+DECLARE_REFLECTION_STRUCT(D3D12_RASTERIZER_DESC1);
+DECLARE_REFLECTION_STRUCT(D3D12_RASTERIZER_DESC2);
 DECLARE_REFLECTION_STRUCT(D3D12_DEPTH_STENCILOP_DESC);
+DECLARE_REFLECTION_STRUCT(D3D12_DEPTH_STENCILOP_DESC1);
 DECLARE_REFLECTION_STRUCT(D3D12_DEPTH_STENCIL_DESC);
+DECLARE_REFLECTION_STRUCT(D3D12_DEPTH_STENCIL_DESC1);
+DECLARE_REFLECTION_STRUCT(D3D12_DEPTH_STENCIL_DESC2);
 DECLARE_REFLECTION_STRUCT(D3D12_INPUT_ELEMENT_DESC);
 DECLARE_REFLECTION_STRUCT(D3D12_INPUT_LAYOUT_DESC);
 DECLARE_REFLECTION_STRUCT(D3D12_RENDER_TARGET_BLEND_DESC);
@@ -710,6 +822,7 @@ DECLARE_REFLECTION_STRUCT(D3D12_INDIRECT_ARGUMENT_DESC);
 DECLARE_REFLECTION_STRUCT(D3D12_COMMAND_SIGNATURE_DESC);
 DECLARE_REFLECTION_STRUCT(D3D12_QUERY_HEAP_DESC);
 DECLARE_REFLECTION_STRUCT(D3D12_SAMPLER_DESC);
+DECLARE_REFLECTION_STRUCT(D3D12_SAMPLER_DESC2);
 DECLARE_REFLECTION_STRUCT(D3D12_CONSTANT_BUFFER_VIEW_DESC);
 DECLARE_REFLECTION_STRUCT(D3D12_BUFFER_SRV);
 DECLARE_REFLECTION_STRUCT(D3D12_TEX1D_SRV);
@@ -743,6 +856,8 @@ DECLARE_REFLECTION_STRUCT(D3D12_TEX1D_UAV);
 DECLARE_REFLECTION_STRUCT(D3D12_TEX1D_ARRAY_UAV);
 DECLARE_REFLECTION_STRUCT(D3D12_TEX2D_UAV);
 DECLARE_REFLECTION_STRUCT(D3D12_TEX2D_ARRAY_UAV);
+DECLARE_REFLECTION_STRUCT(D3D12_TEX2DMS_UAV);
+DECLARE_REFLECTION_STRUCT(D3D12_TEX2DMS_ARRAY_UAV);
 DECLARE_REFLECTION_STRUCT(D3D12_TEX3D_UAV);
 DECLARE_REFLECTION_STRUCT(D3D12_UNORDERED_ACCESS_VIEW_DESC);
 DECLARE_REFLECTION_STRUCT(D3D12_DEPTH_STENCIL_VALUE);
@@ -758,7 +873,6 @@ DECLARE_REFLECTION_STRUCT(D3D12_RECT);
 DECLARE_REFLECTION_STRUCT(D3D12_BOX);
 DECLARE_REFLECTION_STRUCT(D3D12_VIEWPORT);
 DECLARE_REFLECTION_STRUCT(D3D12_RT_FORMAT_ARRAY);
-DECLARE_REFLECTION_STRUCT(D3D12_DEPTH_STENCIL_DESC1);
 DECLARE_REFLECTION_STRUCT(D3D12_VIEW_INSTANCE_LOCATION);
 DECLARE_REFLECTION_STRUCT(D3D12_VIEW_INSTANCING_DESC);
 DECLARE_REFLECTION_STRUCT(D3D12_SAMPLE_POSITION);
@@ -776,12 +890,19 @@ DECLARE_REFLECTION_STRUCT(D3D12_RENDER_PASS_ENDING_ACCESS);
 DECLARE_REFLECTION_STRUCT(D3D12_RENDER_PASS_RENDER_TARGET_DESC);
 DECLARE_REFLECTION_STRUCT(D3D12_RENDER_PASS_DEPTH_STENCIL_DESC);
 DECLARE_REFLECTION_STRUCT(D3D12_STATIC_SAMPLER_DESC);
+DECLARE_REFLECTION_STRUCT(D3D12_STATIC_SAMPLER_DESC1);
 DECLARE_REFLECTION_STRUCT(D3D12_DESCRIPTOR_RANGE1);
 DECLARE_REFLECTION_STRUCT(D3D12_ROOT_DESCRIPTOR_TABLE1);
 DECLARE_REFLECTION_STRUCT(D3D12_ROOT_CONSTANTS);
 DECLARE_REFLECTION_STRUCT(D3D12_ROOT_DESCRIPTOR1);
 DECLARE_REFLECTION_STRUCT(D3D12_RESOURCE_DESC1);
 DECLARE_REFLECTION_STRUCT(D3D12_MIP_REGION);
+DECLARE_REFLECTION_STRUCT(D3D12_BARRIER_SUBRESOURCE_RANGE);
+DECLARE_REFLECTION_STRUCT(D3D12_BUFFER_BARRIER);
+DECLARE_REFLECTION_STRUCT(D3D12_TEXTURE_BARRIER);
+DECLARE_REFLECTION_STRUCT(D3D12_GLOBAL_BARRIER);
+DECLARE_REFLECTION_STRUCT(D3D12_BARRIER_GROUP);
+DECLARE_REFLECTION_STRUCT(D3D12ResourceLayout);
 
 DECLARE_DESERIALISE_TYPE(D3D12_DISCARD_REGION);
 DECLARE_DESERIALISE_TYPE(D3D12_GRAPHICS_PIPELINE_STATE_DESC);
@@ -790,6 +911,7 @@ DECLARE_DESERIALISE_TYPE(D3D12_COMMAND_SIGNATURE_DESC);
 DECLARE_DESERIALISE_TYPE(D3D12_EXPANDED_PIPELINE_STATE_STREAM_DESC);
 DECLARE_DESERIALISE_TYPE(D3D12_RENDER_PASS_RENDER_TARGET_DESC);
 DECLARE_DESERIALISE_TYPE(D3D12_RENDER_PASS_DEPTH_STENCIL_DESC);
+DECLARE_DESERIALISE_TYPE(D3D12_BARRIER_GROUP);
 
 enum class D3D12Chunk : uint32_t
 {
@@ -907,5 +1029,14 @@ enum class D3D12Chunk : uint32_t
   Device_CreatePlacedResource1,
   Device_CreateCommandQueue1,
   CoherentMapWrite,
+  Device_CreateSampler2,
+  Device_CreateCommittedResource3,
+  Device_CreatePlacedResource2,
+  Device_CreateReservedResource1,
+  Device_CreateReservedResource2,
+  List_OMSetFrontAndBackStencilRef,
+  List_RSSetDepthBias,
+  List_IASetIndexBufferStripCutValue,
+  List_Barrier,
   Max,
 };
