@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,13 +24,17 @@
 
 #include <algorithm>
 #include <iterator>
+#include "core/settings.h"
 #include "driver/ihv/amd/amd_counters.h"
+#include "driver/ihv/nv/nv_d3d12_counters.h"
 #include "d3d12_command_list.h"
 #include "d3d12_command_queue.h"
 #include "d3d12_common.h"
 #include "d3d12_device.h"
 #include "d3d12_replay.h"
 
+RDOC_EXTERN_CONFIG(bool, D3D12_Debug_SingleSubmitFlushing);
+
 rdcarray<GPUCounter> D3D12Replay::EnumerateCounters()
 {
   rdcarray<GPUCounter> ret;
@@ -54,6 +58,11 @@ rdcarray<GPUCounter> D3D12Replay::EnumerateCounters()
     ret.append(m_pAMDCounters->GetPublicCounterIds());
   }
 
+  if(m_pNVCounters)
+  {
+    ret.append(m_pNVCounters->EnumerateCounters());
+  }
+
   return ret;
 }
 
@@ -72,6 +81,13 @@ CounterDescription D3D12Replay::DescribeCounter(GPUCounter counterID)
     }
   }
 
+  /////NVIDIA//////
+  if(m_pNVCounters && m_pNVCounters->HasCounter(counterID))
+  {
+    desc = m_pNVCounters->DescribeCounter(counterID);
+    return desc;
+  }
+
   // 0808CC9B-79DF-4549-81F7-85494E648F22
   desc.uuid.words[0] = 0x0808CC9B;
   desc.uuid.words[1] = 0x79DF4549;
@@ -497,7 +513,7 @@ rdcarray<CounterResult> D3D12Replay::FetchCounters(const rdcarray<GPUCounter> &c
 
   rdcarray<GPUCounter> d3dCounters;
   std::copy_if(counters.begin(), counters.end(), std::back_inserter(d3dCounters),
-               [](const GPUCounter &c) { return !IsAMDCounter(c); });
+               [](const GPUCounter &c) { return IsGenericCounter(c); });
 
   if(m_pAMDCounters)
   {
@@ -512,6 +528,19 @@ rdcarray<CounterResult> D3D12Replay::FetchCounters(const rdcarray<GPUCounter> &c
     }
   }
 
+  if(m_pNVCounters)
+  {
+    // Filter out the NVIDIA counters
+    rdcarray<GPUCounter> nvCounters;
+    std::copy_if(counters.begin(), counters.end(), std::back_inserter(nvCounters),
+                 [=](const GPUCounter &c) { return m_pNVCounters->HasCounter(c); });
+    if(!nvCounters.empty())
+    {
+      rdcarray<CounterResult> results = m_pNVCounters->FetchCounters(nvCounters, *m_pDevice);
+      ret.append(results);
+    }
+  }
+
   if(d3dCounters.empty())
   {
     return ret;
@@ -608,10 +637,11 @@ rdcarray<CounterResult> D3D12Replay::FetchCounters(const rdcarray<GPUCounter> &c
   // replay the events to perform all the queries
   m_pDevice->ReplayLog(0, maxEID, eReplay_Full);
 
-#if ENABLED(SINGLE_FLUSH_VALIDATE)
-  m_pDevice->ExecuteLists();
-  m_pDevice->FlushLists(true);
-#endif
+  if(D3D12_Debug_SingleSubmitFlushing())
+  {
+    m_pDevice->ExecuteLists();
+    m_pDevice->FlushLists(true);
+  }
 
   // Only supported with developer mode drivers!!!
   m_pDevice->SetStablePowerState(FALSE);
@@ -640,6 +670,7 @@ rdcarray<CounterResult> D3D12Replay::FetchCounters(const rdcarray<GPUCounter> &c
 
   m_pDevice->ExecuteLists();
   m_pDevice->FlushLists();
+  m_pDevice->GPUSyncAllQueues();
 
   D3D12_RANGE range;
   range.Begin = 0;
