@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -153,7 +153,7 @@ public:
 
   void FillWithDiscardPattern(ID3D12GraphicsCommandListX *cmd, const D3D12RenderState &state,
                               DiscardType type, ID3D12Resource *res,
-                              const D3D12_DISCARD_REGION *region);
+                              const D3D12_DISCARD_REGION *region, D3D12_BARRIER_LAYOUT LayoutAfter);
 
   D3D12_CPU_DESCRIPTOR_HANDLE GetCPUHandle(CBVUAVSRVSlot slot);
   D3D12_CPU_DESCRIPTOR_HANDLE GetCPUHandle(RTVSlot slot);
@@ -173,7 +173,7 @@ public:
   D3D12_CPU_DESCRIPTOR_HANDLE GetUAVClearHandle(CBVUAVSRVSlot slot);
 
   void PrepareTextureSampling(ID3D12Resource *resource, CompType typeCast, int &resType,
-                              rdcarray<D3D12_RESOURCE_BARRIER> &barriers);
+                              BarrierSet &barrierSet);
 
   MeshDisplayPipelines CacheMeshDisplayPipelines(const MeshFormat &primary,
                                                  const MeshFormat &secondary);
@@ -245,17 +245,17 @@ private:
   // Discard pattern rendering
   ID3DBlob *m_DiscardFloatPS = NULL;
   ID3DBlob *m_DiscardIntPS = NULL;
-  ID3D12Resource *m_DiscardConstants = NULL;
+  ID3D12Resource *m_DiscardConstantsDiscard = NULL;
+  ID3D12Resource *m_DiscardConstantsUndefined = NULL;
   ID3D12RootSignature *m_DiscardRootSig = NULL;
 
   std::map<rdcpair<DXGI_FORMAT, UINT>, ID3D12PipelineState *> m_DiscardPipes;
-  std::map<DXGI_FORMAT, ID3D12Resource *> m_DiscardPatterns;
+  std::map<rdcpair<DiscardType, DXGI_FORMAT>, ID3D12Resource *> m_DiscardPatterns;
   rdcarray<ID3D12Resource *> m_DiscardBuffers;
 };
 
-void MoveRootSignatureElementsToRegisterSpace(D3D12RootSignature &sig, uint32_t registerSpace,
-                                              D3D12DescriptorType type,
-                                              D3D12_SHADER_VISIBILITY visibility);
+uint32_t GetFreeRegSpace(const D3D12RootSignature &sig, const uint32_t registerSpace,
+                         D3D12DescriptorType type, D3D12_SHADER_VISIBILITY visibility);
 
 void AddDebugDescriptorsToRenderState(WrappedID3D12Device *pDevice, D3D12RenderState &rs,
                                       const rdcarray<PortableHandle> &handles,
