@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -34,6 +34,8 @@ class AMDCounters;
 struct D3D12AMDActionCallback;
 class WrappedID3D12Device;
 
+class NVD3D12Counters;
+
 class D3D12DebugManager;
 
 struct PortableHandle;
@@ -189,12 +191,12 @@ public:
 
   rdcarray<ShaderEncoding> GetCustomShaderEncodings()
   {
-    return {ShaderEncoding::DXBC, ShaderEncoding::HLSL};
+    return {ShaderEncoding::DXBC, ShaderEncoding::DXIL, ShaderEncoding::HLSL};
   }
   rdcarray<ShaderSourcePrefix> GetCustomShaderSourcePrefixes();
   rdcarray<ShaderEncoding> GetTargetShaderEncodings()
   {
-    return {ShaderEncoding::DXBC, ShaderEncoding::HLSL};
+    return {ShaderEncoding::DXBC, ShaderEncoding::DXIL, ShaderEncoding::HLSL};
   }
   void BuildTargetShader(ShaderEncoding sourceEncoding, const bytebuf &source, const rdcstr &entry,
                          const ShaderCompileFlags &compileFlags, ShaderStage type, ResourceId &id,
@@ -254,19 +256,20 @@ public:
   bool IsRenderOutput(ResourceId id) { return GetRenderOutputSubresource(id).mip != ~0U; }
   void FileChanged() {}
   AMDCounters *GetAMDCounters() { return m_pAMDCounters; }
-  void PatchQuadWritePS(D3D12_EXPANDED_PIPELINE_STATE_STREAM_DESC &pipeDesc, bool dxil);
+  void PatchQuadWritePS(D3D12_EXPANDED_PIPELINE_STATE_STREAM_DESC &pipeDesc, uint32_t regSpace,
+                        bool dxil);
 
 private:
   void FillRootElements(uint32_t eventId, const D3D12RenderState::RootSignature &rootSig,
                         const ShaderBindpointMapping *mappings[(uint32_t)ShaderStage::Count],
                         rdcarray<D3D12Pipe::RootSignatureRange> &rootElements);
   void FillResourceView(D3D12Pipe::View &view, const D3D12Descriptor *desc);
-  void FillSampler(D3D12Pipe::Sampler &view, const D3D12_SAMPLER_DESC &desc);
+  void FillSampler(D3D12Pipe::Sampler &view, const D3D12_SAMPLER_DESC2 &desc);
 
   bool CreateSOBuffers();
   void ClearPostVSCache();
 
-  void FetchShaderFeedback(uint32_t eventId);
+  bool FetchShaderFeedback(uint32_t eventId);
   void ClearFeedbackCache();
 
   void RefreshDerivedReplacements();
@@ -293,6 +296,7 @@ private:
   struct Feedback
   {
     ID3D12Resource *FeedbackBuffer = NULL;
+    ID3D12QueryHeap *PipeStatsHeap = NULL;
 
     std::unordered_map<uint32_t, D3D12DynamicShaderFeedback> Usage;
   } m_BindlessFeedback;
@@ -327,6 +331,8 @@ private:
 
       float nearPlane = 0.0f;
       float farPlane = 0.0f;
+
+      rdcstr status;
     } vsin, vsout, gsout;
 
     const StageData &GetStage(MeshDataStage type)
@@ -503,6 +509,7 @@ private:
   std::map<rdcfixedarray<uint32_t, 4>, bytebuf> m_PatchedPSCache;
 
   void FillTimersAMD(uint32_t *eventStartID, uint32_t *sampleIndex, rdcarray<uint32_t> *eventIDs);
-
   rdcarray<CounterResult> FetchCountersAMD(const rdcarray<GPUCounter> &counters);
+
+  NVD3D12Counters *m_pNVCounters = NULL;
 };
