@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -1195,7 +1195,7 @@ bool GLResourceManager::Serialise_InitialState(SerialiserType &ser, ResourceId i
 
         const auto &shadDetails = m_Driver->m_Shaders[details.stageShaders[i]];
 
-        IsProgramSPIRV |= shadDetails.reflection->encoding == ShaderEncoding::SPIRV;
+        IsProgramSPIRV |= shadDetails.reflection->encoding == ShaderEncoding::OpenGLSPIRV;
 
         GLuint shad = drv.glCreateShader(shadDetails.type);
 
@@ -1224,9 +1224,18 @@ bool GLResourceManager::Serialise_InitialState(SerialiserType &ser, ResourceId i
           for(size_t s = 0; s < shadDetails.sources.size(); s++)
             srcs[s] = (char *)shadDetails.sources[s].c_str();
           drv.glShaderSource(shad, (GLsizei)shadDetails.sources.size(), srcs, NULL);
-
           SAFE_DELETE_ARRAY(srcs);
-          drv.glCompileShader(shad);
+
+          char **includes = new char *[shadDetails.includepaths.size()];
+          for(size_t s = 0; s < shadDetails.includepaths.size(); s++)
+            includes[s] = (char *)shadDetails.includepaths[s].c_str();
+
+          if(shadDetails.includepaths.empty())
+            drv.glCompileShader(shad);
+          else
+            drv.glCompileShaderIncludeARB(shad, (GLsizei)shadDetails.includepaths.size(), includes,
+                                          NULL);
+          SAFE_DELETE_ARRAY(includes);
           drv.glAttachShader(initProg, shad);
           drv.glDeleteShader(shad);
         }
@@ -2177,34 +2186,37 @@ void GLResourceManager::Apply_InitialState(GLResource live, const GLInitialConte
 
       GLenum fmt = details.internalFormat;
 
-      // update width from here as it's authoratitive - the texture might have been resized in
-      // multiple rebinds that we will not have serialised before.
-      details.width =
-          state.texBufSize / uint32_t(GetByteSize(1, 1, 1, GetBaseFormat(fmt), GetDataType(fmt)));
+      if(buffer && fmt != eGL_NONE)
+      {
+        // update width from here as it's authoratitive - the texture might have been resized in
+        // multiple rebinds that we will not have serialised before.
+        details.width =
+            state.texBufSize / uint32_t(GetByteSize(1, 1, 1, GetBaseFormat(fmt), GetDataType(fmt)));
 
-      if(GL.glTextureBufferRangeEXT && (state.texBufOffs > 0 || state.texBufSize > 0))
-      {
-        // restore texbuffer only state
-        GL.glTextureBufferRangeEXT(live.name, eGL_TEXTURE_BUFFER, details.internalFormat, buffer,
-                                   state.texBufOffs, state.texBufSize);
-      }
-      else
-      {
-        uint32_t bufSize = 0;
-        GL.glGetNamedBufferParameterivEXT(buffer, eGL_BUFFER_SIZE, (GLint *)&bufSize);
-        if(state.texBufOffs > 0 || state.texBufSize > bufSize)
+        if(GL.glTextureBufferRangeEXT && (state.texBufOffs > 0 || state.texBufSize > 0))
         {
-          const char *msg =
-              "glTextureBufferRangeEXT is not supported on your GL implementation, but is needed "
-              "for correct replay.\n"
-              "The original capture created a texture buffer with a range - replay will use the "
-              "whole buffer, which is likely incorrect.";
-          RDCERR("%s", msg);
-          m_Driver->AddDebugMessage(MessageCategory::Resource_Manipulation, MessageSeverity::High,
-                                    MessageSource::IncorrectAPIUse, msg);
+          // restore texbuffer only state
+          GL.glTextureBufferRangeEXT(live.name, eGL_TEXTURE_BUFFER, details.internalFormat, buffer,
+                                     state.texBufOffs, state.texBufSize);
         }
+        else
+        {
+          uint32_t bufSize = 0;
+          GL.glGetNamedBufferParameterivEXT(buffer, eGL_BUFFER_SIZE, (GLint *)&bufSize);
+          if(state.texBufOffs > 0 || state.texBufSize > bufSize)
+          {
+            const char *msg =
+                "glTextureBufferRangeEXT is not supported on your GL implementation, but is needed "
+                "for correct replay.\n"
+                "The original capture created a texture buffer with a range - replay will use the "
+                "whole buffer, which is likely incorrect.";
+            RDCERR("%s", msg);
+            m_Driver->AddDebugMessage(MessageCategory::Resource_Manipulation, MessageSeverity::High,
+                                      MessageSource::IncorrectAPIUse, msg);
+          }
 
-        GL.glTextureBufferEXT(live.name, eGL_TEXTURE_BUFFER, details.internalFormat, buffer);
+          GL.glTextureBufferEXT(live.name, eGL_TEXTURE_BUFFER, details.internalFormat, buffer);
+        }
       }
     }
   }
