@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -492,7 +492,7 @@ void GLReplay::InitPostVSBuffers(uint32_t eventId)
     }
   }
 
-  if(vsRefl->encoding == ShaderEncoding::SPIRV)
+  if(vsRefl->encoding == ShaderEncoding::OpenGLSPIRV)
   {
     // SPIR-V path
     vsOrigShader = stageShaders[0];
@@ -1189,7 +1189,7 @@ void GLReplay::InitPostVSBuffers(uint32_t eventId)
       lastIndex = 2;
     }
 
-    bool lastSPIRV = (lastRefl->encoding == ShaderEncoding::SPIRV);
+    bool lastSPIRV = (lastRefl->encoding == ShaderEncoding::OpenGLSPIRV);
 
     RDCASSERT(lastRefl);
 
