@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2022 Baldur Karlsson
+ * Copyright (c) 2022-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -1034,6 +1034,7 @@ void WrappedMTLDevice::RegisterDrawableInfo(CA::MetalDrawable *caMtlDrawable)
   MetalDrawableInfo drawableInfo;
   drawableInfo.mtlLayer = caMtlDrawable->layer();
   drawableInfo.texture = GetWrapped(caMtlDrawable->texture());
+  drawableInfo.drawableID = caMtlDrawable->drawableID();
   SCOPED_LOCK(m_CaptureDrawablesLock);
   RDCASSERTEQUAL(m_CaptureDrawableInfos.find(caMtlDrawable), m_CaptureDrawableInfos.end());
   m_CaptureDrawableInfos[caMtlDrawable] = drawableInfo;
@@ -1052,6 +1053,17 @@ MetalDrawableInfo WrappedMTLDevice::UnregisterDrawableInfo(MTL::Drawable *mtlDra
       return drawableInfo;
     }
   }
+  // Not found by pointer fall back and check by drawableID
+  NS::UInteger drawableID = mtlDrawable->drawableID();
+  for(auto it = m_CaptureDrawableInfos.begin(); it != m_CaptureDrawableInfos.end(); ++it)
+  {
+    drawableInfo = it->second;
+    if(drawableInfo.drawableID == drawableID)
+    {
+      m_CaptureDrawableInfos.erase(it);
+      return drawableInfo;
+    }
+  }
   drawableInfo.mtlLayer = NULL;
   drawableInfo.texture = NULL;
   return drawableInfo;
