@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2022 Baldur Karlsson
+ * Copyright (c) 2022-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -63,7 +63,7 @@
 }
 
 // MTLCommandEncoder : based on the protocol defined in
-// Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX12.1.sdk/System/Library/Frameworks/Metal.framework/Headers/MTLCommandEncoder.h
+// Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX13.1.sdk/System/Library/Frameworks/Metal.framework/Headers/MTLCommandEncoder.h
 
 - (id<MTLDevice>)device
 {
@@ -191,16 +191,13 @@
                                  withRange:range];
 }
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (void)setVertexVisibleFunctionTable:(nullable id<MTLVisibleFunctionTable>)functionTable
                         atBufferIndex:(NSUInteger)bufferIndex API_AVAILABLE(macos(12.0), ios(15.0))
 {
   METAL_NOT_HOOKED();
   return [self.real setVertexVisibleFunctionTable:functionTable atBufferIndex:bufferIndex];
 }
-#endif
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (void)setVertexVisibleFunctionTables:
             (const id<MTLVisibleFunctionTable> __nullable[__nonnull])functionTables
                        withBufferRange:(NSRange)range API_AVAILABLE(macos(12.0), ios(15.0))
@@ -208,9 +205,7 @@
   METAL_NOT_HOOKED();
   return [self.real setVertexVisibleFunctionTables:functionTables withBufferRange:range];
 }
-#endif
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (void)setVertexIntersectionFunctionTable:
             (nullable id<MTLIntersectionFunctionTable>)intersectionFunctionTable
                              atBufferIndex:(NSUInteger)bufferIndex
@@ -220,27 +215,22 @@
   return [self.real setVertexIntersectionFunctionTable:intersectionFunctionTable
                                          atBufferIndex:bufferIndex];
 }
-#endif
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (void)setVertexIntersectionFunctionTables:
-            (const id<MTLIntersectionFunctionTable> __nullable[__nonnull])intersectionFunctionTable
+            (const id<MTLIntersectionFunctionTable> __nullable[__nonnull])intersectionFunctionTables
                             withBufferRange:(NSRange)range API_AVAILABLE(macos(12.0), ios(15.0))
 {
   METAL_NOT_HOOKED();
-  return [self.real setVertexIntersectionFunctionTables:intersectionFunctionTable
+  return [self.real setVertexIntersectionFunctionTables:intersectionFunctionTables
                                         withBufferRange:range];
 }
-#endif
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (void)setVertexAccelerationStructure:(nullable id<MTLAccelerationStructure>)accelerationStructure
                          atBufferIndex:(NSUInteger)bufferIndex API_AVAILABLE(macos(12.0), ios(15.0))
 {
   METAL_NOT_HOOKED();
   return [self.real setVertexAccelerationStructure:accelerationStructure atBufferIndex:bufferIndex];
 }
-#endif
 
 - (void)setViewport:(MTLViewport)viewport
 {
@@ -384,16 +374,13 @@
                                    withRange:range];
 }
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (void)setFragmentVisibleFunctionTable:(nullable id<MTLVisibleFunctionTable>)functionTable
                           atBufferIndex:(NSUInteger)bufferIndex API_AVAILABLE(macos(12.0), ios(15.0))
 {
   METAL_NOT_HOOKED();
   return [self.real setFragmentVisibleFunctionTable:functionTable atBufferIndex:bufferIndex];
 }
-#endif
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (void)setFragmentVisibleFunctionTables:
             (const id<MTLVisibleFunctionTable> __nullable[__nonnull])functionTables
                          withBufferRange:(NSRange)range API_AVAILABLE(macos(12.0), ios(15.0))
@@ -401,9 +388,7 @@
   METAL_NOT_HOOKED();
   return [self.real setFragmentVisibleFunctionTables:functionTables withBufferRange:range];
 }
-#endif
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (void)setFragmentIntersectionFunctionTable:
             (nullable id<MTLIntersectionFunctionTable>)intersectionFunctionTable
                                atBufferIndex:(NSUInteger)bufferIndex
@@ -413,20 +398,16 @@
   return [self.real setFragmentIntersectionFunctionTable:intersectionFunctionTable
                                            atBufferIndex:bufferIndex];
 }
-#endif
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (void)setFragmentIntersectionFunctionTables:
-            (const id<MTLIntersectionFunctionTable> __nullable[__nonnull])intersectionFunctionTable
+            (const id<MTLIntersectionFunctionTable> __nullable[__nonnull])intersectionFunctionTables
                               withBufferRange:(NSRange)range API_AVAILABLE(macos(12.0), ios(15.0))
 {
   METAL_NOT_HOOKED();
-  return [self.real setFragmentIntersectionFunctionTables:intersectionFunctionTable
+  return [self.real setFragmentIntersectionFunctionTables:intersectionFunctionTables
                                           withBufferRange:range];
 }
-#endif
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (void)setFragmentAccelerationStructure:(nullable id<MTLAccelerationStructure>)accelerationStructure
                            atBufferIndex:(NSUInteger)bufferIndex
     API_AVAILABLE(macos(12.0), ios(15.0))
@@ -435,7 +416,6 @@
   return
       [self.real setFragmentAccelerationStructure:accelerationStructure atBufferIndex:bufferIndex];
 }
-#endif
 
 - (void)setBlendColorRed:(float)red green:(float)green blue:(float)blue alpha:(float)alpha
 {
@@ -511,6 +491,262 @@
   return [self.real setStencilStoreActionOptions:storeActionOptions];
 }
 
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setObjectBytes:(const void *)bytes
+                length:(NSUInteger)length
+               atIndex:(NSUInteger)index API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setObjectBytes:bytes length:length atIndex:index];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setObjectBuffer:(nullable id<MTLBuffer>)buffer
+                 offset:(NSUInteger)offset
+                atIndex:(NSUInteger)index API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setObjectBuffer:buffer offset:offset atIndex:index];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setObjectBufferOffset:(NSUInteger)offset
+                      atIndex:(NSUInteger)index API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setObjectBufferOffset:offset atIndex:index];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setObjectBuffers:(const id<MTLBuffer> __nullable[__nonnull])buffers
+                 offsets:(const NSUInteger[__nonnull])offsets
+               withRange:(NSRange)range API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setObjectBuffers:buffers offsets:offsets withRange:range];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setObjectTexture:(nullable id<MTLTexture>)texture
+                 atIndex:(NSUInteger)index API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setObjectTexture:texture atIndex:index];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setObjectTextures:(const id<MTLTexture> __nullable[__nonnull])textures
+                withRange:(NSRange)range API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setObjectTextures:textures withRange:range];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setObjectSamplerState:(nullable id<MTLSamplerState>)sampler
+                      atIndex:(NSUInteger)index API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setObjectSamplerState:sampler atIndex:index];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setObjectSamplerStates:(const id<MTLSamplerState> __nullable[__nonnull])samplers
+                     withRange:(NSRange)range API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setObjectSamplerStates:samplers withRange:range];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setObjectSamplerState:(nullable id<MTLSamplerState>)sampler
+                  lodMinClamp:(float)lodMinClamp
+                  lodMaxClamp:(float)lodMaxClamp
+                      atIndex:(NSUInteger)index API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setObjectSamplerState:sampler
+                              lodMinClamp:lodMinClamp
+                              lodMaxClamp:lodMaxClamp
+                                  atIndex:index];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setObjectSamplerStates:(const id<MTLSamplerState> __nullable[__nonnull])samplers
+                  lodMinClamps:(const float[__nonnull])lodMinClamps
+                  lodMaxClamps:(const float[__nonnull])lodMaxClamps
+                     withRange:(NSRange)range API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setObjectSamplerStates:samplers
+                              lodMinClamps:lodMinClamps
+                              lodMaxClamps:lodMaxClamps
+                                 withRange:range];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setObjectThreadgroupMemoryLength:(NSUInteger)length
+                                 atIndex:(NSUInteger)index API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setObjectThreadgroupMemoryLength:length atIndex:index];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setMeshBytes:(const void *)bytes
+              length:(NSUInteger)length
+             atIndex:(NSUInteger)index API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setMeshBytes:bytes length:length atIndex:index];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setMeshBuffer:(nullable id<MTLBuffer>)buffer
+               offset:(NSUInteger)offset
+              atIndex:(NSUInteger)index API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setMeshBuffer:buffer offset:offset atIndex:index];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setMeshBufferOffset:(NSUInteger)offset
+                    atIndex:(NSUInteger)index API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setMeshBufferOffset:offset atIndex:index];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setMeshBuffers:(const id<MTLBuffer> __nullable[__nonnull])buffers
+               offsets:(const NSUInteger[__nonnull])offsets
+             withRange:(NSRange)range API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setMeshBuffers:buffers offsets:offsets withRange:range];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setMeshTexture:(nullable id<MTLTexture>)texture
+               atIndex:(NSUInteger)index API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setMeshTexture:texture atIndex:index];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setMeshTextures:(const id<MTLTexture> __nullable[__nonnull])textures
+              withRange:(NSRange)range API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setMeshTextures:textures withRange:range];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setMeshSamplerState:(nullable id<MTLSamplerState>)sampler
+                    atIndex:(NSUInteger)index API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setMeshSamplerState:sampler atIndex:index];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setMeshSamplerStates:(const id<MTLSamplerState> __nullable[__nonnull])samplers
+                   withRange:(NSRange)range API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setMeshSamplerStates:samplers withRange:range];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setMeshSamplerState:(nullable id<MTLSamplerState>)sampler
+                lodMinClamp:(float)lodMinClamp
+                lodMaxClamp:(float)lodMaxClamp
+                    atIndex:(NSUInteger)index API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setMeshSamplerState:sampler
+                            lodMinClamp:lodMinClamp
+                            lodMaxClamp:lodMaxClamp
+                                atIndex:index];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)setMeshSamplerStates:(const id<MTLSamplerState> __nullable[__nonnull])samplers
+                lodMinClamps:(const float[__nonnull])lodMinClamps
+                lodMaxClamps:(const float[__nonnull])lodMaxClamps
+                   withRange:(NSRange)range API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real setMeshSamplerStates:samplers
+                            lodMinClamps:lodMinClamps
+                            lodMaxClamps:lodMaxClamps
+                               withRange:range];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)drawMeshThreadgroups:(MTLSize)threadgroupsPerGrid
+    threadsPerObjectThreadgroup:(MTLSize)threadsPerObjectThreadgroup
+      threadsPerMeshThreadgroup:(MTLSize)threadsPerMeshThreadgroup
+    API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real drawMeshThreadgroups:threadgroupsPerGrid
+             threadsPerObjectThreadgroup:threadsPerObjectThreadgroup
+               threadsPerMeshThreadgroup:threadsPerMeshThreadgroup];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)drawMeshThreads:(MTLSize)threadsPerGrid
+    threadsPerObjectThreadgroup:(MTLSize)threadsPerObjectThreadgroup
+      threadsPerMeshThreadgroup:(MTLSize)threadsPerMeshThreadgroup
+    API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real drawMeshThreads:threadsPerGrid
+        threadsPerObjectThreadgroup:threadsPerObjectThreadgroup
+          threadsPerMeshThreadgroup:threadsPerMeshThreadgroup];
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (void)drawMeshThreadgroupsWithIndirectBuffer:(id<MTLBuffer>)indirectBuffer
+                          indirectBufferOffset:(NSUInteger)indirectBufferOffset
+                   threadsPerObjectThreadgroup:(MTLSize)threadsPerObjectThreadgroup
+                     threadsPerMeshThreadgroup:(MTLSize)threadsPerMeshThreadgroup
+    API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  METAL_NOT_HOOKED();
+  return [self.real drawMeshThreadgroupsWithIndirectBuffer:indirectBuffer
+                                      indirectBufferOffset:indirectBufferOffset
+                               threadsPerObjectThreadgroup:threadsPerObjectThreadgroup
+                                 threadsPerMeshThreadgroup:threadsPerMeshThreadgroup];
+}
+#endif
+
 - (void)drawPrimitives:(MTLPrimitiveType)primitiveType
            vertexStart:(NSUInteger)vertexStart
            vertexCount:(NSUInteger)vertexCount
@@ -829,25 +1065,20 @@
                                withRange:range];
 }
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (void)setTileVisibleFunctionTable:(nullable id<MTLVisibleFunctionTable>)functionTable
                       atBufferIndex:(NSUInteger)bufferIndex API_AVAILABLE(macos(12.0), ios(15.0))
 {
   METAL_NOT_HOOKED();
   return [self.real setTileVisibleFunctionTable:functionTable atBufferIndex:bufferIndex];
 }
-#endif
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (void)setTileVisibleFunctionTables:(const id<MTLVisibleFunctionTable> __nullable[__nonnull])functionTables
                      withBufferRange:(NSRange)range API_AVAILABLE(macos(12.0), ios(15.0))
 {
   METAL_NOT_HOOKED();
   return [self.real setTileVisibleFunctionTables:functionTables withBufferRange:range];
 }
-#endif
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (void)setTileIntersectionFunctionTable:
             (nullable id<MTLIntersectionFunctionTable>)intersectionFunctionTable
                            atBufferIndex:(NSUInteger)bufferIndex
@@ -857,9 +1088,7 @@
   return [self.real setTileIntersectionFunctionTable:intersectionFunctionTable
                                        atBufferIndex:bufferIndex];
 }
-#endif
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (void)setTileIntersectionFunctionTables:
             (const id<MTLIntersectionFunctionTable> __nullable[__nonnull])intersectionFunctionTable
                           withBufferRange:(NSRange)range API_AVAILABLE(macos(12.0), ios(15.0))
@@ -868,16 +1097,13 @@
   return
       [self.real setTileIntersectionFunctionTables:intersectionFunctionTable withBufferRange:range];
 }
-#endif
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (void)setTileAccelerationStructure:(nullable id<MTLAccelerationStructure>)accelerationStructure
                        atBufferIndex:(NSUInteger)bufferIndex API_AVAILABLE(macos(12.0), ios(15.0))
 {
   METAL_NOT_HOOKED();
   return [self.real setTileAccelerationStructure:accelerationStructure atBufferIndex:bufferIndex];
 }
-#endif
 
 - (void)dispatchThreadsPerTile:(MTLSize)threadsPerTile
     API_AVAILABLE(macos(11.0), macCatalyst(14.0), ios(11.0), tvos(14.5))
@@ -976,7 +1202,7 @@
 - (void)memoryBarrierWithScope:(MTLBarrierScope)scope
                    afterStages:(MTLRenderStages)after
                   beforeStages:(MTLRenderStages)before
-    API_AVAILABLE(macos(10.14), macCatalyst(13.0))API_UNAVAILABLE(ios)
+    API_AVAILABLE(macos(10.14), macCatalyst(13.0), ios(16.0))
 {
   METAL_NOT_HOOKED();
   return [self.real memoryBarrierWithScope:scope afterStages:after beforeStages:before];
@@ -986,7 +1212,7 @@
                              count:(NSUInteger)count
                        afterStages:(MTLRenderStages)after
                       beforeStages:(MTLRenderStages)before
-    API_AVAILABLE(macos(10.14), macCatalyst(13.0))API_UNAVAILABLE(ios)
+    API_AVAILABLE(macos(10.14), macCatalyst(13.0), ios(16.0))
 {
   METAL_NOT_HOOKED();
   return [self.real memoryBarrierWithResources:resources
