@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2022 Baldur Karlsson
+ * Copyright (c) 2022-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -63,7 +63,7 @@
 }
 
 // MTLRenderPipelineState : based on the protocol defined in
-// Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX12.1.sdk/System/Library/Frameworks/Metal.framework/Headers/MTLRenderPipeline.h
+// Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX13.1.sdk/System/Library/Frameworks/Metal.framework/Headers/MTLRenderPipeline.h
 
 - (nullable NSString *)label
 {
@@ -104,7 +104,48 @@
   return self.real.supportIndirectCommandBuffers;
 }
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (NSUInteger)maxTotalThreadsPerObjectThreadgroup API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  return self.real.maxTotalThreadsPerObjectThreadgroup;
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (NSUInteger)maxTotalThreadsPerMeshThreadgroup API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  return self.real.maxTotalThreadsPerMeshThreadgroup;
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (NSUInteger)objectThreadExecutionWidth API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  return self.real.objectThreadExecutionWidth;
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (NSUInteger)meshThreadExecutionWidth API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  return self.real.meshThreadExecutionWidth;
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (NSUInteger)maxTotalThreadgroupsPerMeshGrid API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  return self.real.maxTotalThreadgroupsPerMeshGrid;
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (MTLResourceID)gpuResourceID API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  return self.real.gpuResourceID;
+}
+#endif
+
 - (nullable id<MTLFunctionHandle>)functionHandleWithFunction:(id<MTLFunction>)function
                                                        stage:(MTLRenderStages)stage
     API_AVAILABLE(macos(12.0), ios(15.0))
@@ -112,9 +153,7 @@
   METAL_NOT_HOOKED();
   return [self.real functionHandleWithFunction:function stage:stage];
 }
-#endif
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (nullable id<MTLVisibleFunctionTable>)newVisibleFunctionTableWithDescriptor:
                                             (MTLVisibleFunctionTableDescriptor *__nonnull)descriptor
                                                                         stage:(MTLRenderStages)stage
@@ -123,9 +162,7 @@
   METAL_NOT_HOOKED();
   return [self.real newVisibleFunctionTableWithDescriptor:descriptor stage:stage];
 }
-#endif
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (nullable id<MTLIntersectionFunctionTable>)
 newIntersectionFunctionTableWithDescriptor:(MTLIntersectionFunctionTableDescriptor *_Nonnull)descriptor
                                      stage:(MTLRenderStages)stage
@@ -134,9 +171,7 @@ newIntersectionFunctionTableWithDescriptor:(MTLIntersectionFunctionTableDescript
   METAL_NOT_HOOKED();
   return [self.real newIntersectionFunctionTableWithDescriptor:descriptor stage:stage];
 }
-#endif
 
-#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_0
 - (nullable id<MTLRenderPipelineState>)
 newRenderPipelineStateWithAdditionalBinaryFunctions:
     (nonnull MTLRenderPipelineFunctionsDescriptor *)additionalBinaryFunctions
@@ -147,6 +182,5 @@ newRenderPipelineStateWithAdditionalBinaryFunctions:
   return [self.real newRenderPipelineStateWithAdditionalBinaryFunctions:additionalBinaryFunctions
                                                                   error:error];
 }
-#endif
 
 @end
