@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2022 Baldur Karlsson
+ * Copyright (c) 2022-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -63,7 +63,7 @@
 }
 
 // MTLResource : based on the protocol defined in
-// Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX12.1.sdk/System/Library/Frameworks/Metal.framework/Headers/MTLResource.h
+// Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX13.1.sdk/System/Library/Frameworks/Metal.framework/Headers/MTLResource.h
 
 - (nullable NSString *)label
 {
@@ -260,6 +260,20 @@
   return self.real.allowGPUOptimizedContents;
 }
 
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_12_5
+- (MTLTextureCompressionType)compressionType API_AVAILABLE(macos(12.5), ios(15.0))
+{
+  return self.real.compressionType;
+}
+#endif
+
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= __MAC_13_0
+- (MTLResourceID)gpuResourceID API_AVAILABLE(macos(13.0), ios(16.0))
+{
+  return self.real.gpuResourceID;
+}
+#endif
+
 - (void)getBytes:(void *)pixelBytes
       bytesPerRow:(NSUInteger)bytesPerRow
     bytesPerImage:(NSUInteger)bytesPerImage
