@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -358,11 +358,19 @@ CBufferVariableType DXBCContainer::ParseRDEFType(const RDEFHeader *h, const byte
     case DXBC::VARTYPE_BOOL: ret.varType = VarType::Bool; break;
     case DXBC::VARTYPE_MIN16UINT:
     case DXBC::VARTYPE_UINT: ret.varType = VarType::UInt; break;
+    case DXBC::VARTYPE_INT64:
+    case DXBC::VARTYPE_UINT64:
     case DXBC::VARTYPE_DOUBLE: ret.varType = VarType::Double; break;
     case DXBC::VARTYPE_FLOAT:
     case DXBC::VARTYPE_MIN8FLOAT:
     case DXBC::VARTYPE_MIN10FLOAT:
     case DXBC::VARTYPE_MIN16FLOAT:
+      ret.varType = VarType::Float;
+      break;
+    // new types are actually 16-bit, though alignment is still the same as 32-bit
+    case DXBC::VARTYPE_INT16: ret.varType = VarType::SShort; break;
+    case DXBC::VARTYPE_UINT16: ret.varType = VarType::UShort; break;
+    case DXBC::VARTYPE_FLOAT16: ret.varType = VarType::Half; break;
     default: ret.varType = VarType::Float; break;
   }
 
@@ -567,6 +575,7 @@ const rdcstr &DXBCContainer::GetDisassembly()
     {
       m_Disassembly.clear();
 
+#if DISABLED(DXC_COMPATIBLE_DISASM)
       if(m_GlobalFlags != GlobalShaderFlags::None)
         m_Disassembly += globalFlagsString;
 
@@ -581,6 +590,7 @@ const rdcstr &DXBCContainer::GetDisassembly()
       for(size_t i = 0; i < sizeof(m_Hash); i++)
         m_Disassembly += StringFormat::Fmt("%02x", hashBytes[i]);
       m_Disassembly += "\n\n";
+#endif
 
       m_Disassembly += m_DXILByteCode->GetDisassembly();
     }
