@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -91,9 +91,7 @@ static ShaderConstantType MakeShaderConstantType(bool cbufferPacking, DXBC::CBuf
         stride = 0;
     }
 
-    RDCASSERTMSG("Stride is too large for uint16_t", stride <= 0xffff);
-
-    ret.arrayByteStride = RDCMIN(stride, 0xffffu) & 0xffff;
+    ret.arrayByteStride = stride;
     // in D3D only cbuffers have 16-byte aligned structs
     if(cbufferPacking)
       ret.arrayByteStride = AlignUp16(ret.arrayByteStride);
