@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2020-2022 Baldur Karlsson
+ * Copyright (c) 2020-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,20 +29,27 @@
 
 namespace DXIL
 {
-bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Metadata &meta)
+// DXIL is SO AWFUL. There is an svbr encoding used for the bitcode which negates and shifts. This
+// encoding bitwise-nots and shifts, for no reason?
+static int64_t debug_only_svbr(uint64_t val)
+{
+  if(val & 0x1)
+    return int64_t(~(val >> 1));
+  return val >> 1;
+}
+
+bool Program::ParseDebugMetaRecord(MetadataList &metadata, const LLVMBC::BlockOrRecord &metaRecord,
+                                   Metadata &meta)
 {
   LLVMBC::MetaDataRecord id = (LLVMBC::MetaDataRecord)metaRecord.id;
 
-#define getNonNullMeta(id) &m_Metadata[size_t(id)]
-#define getMeta(id) (id ? &m_Metadata[size_t(id - 1)] : NULL)
-#define getMetaString(id) (id ? &m_Metadata[size_t(id - 1)].str : NULL)
-
   if(id == LLVMBC::MetaDataRecord::FILE)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
-    meta.dwarf = new DIFile(getMeta(metaRecord.ops[1]), getMeta(metaRecord.ops[2]));
-    meta.children = {getMeta(metaRecord.ops[1]), getMeta(metaRecord.ops[2])};
+    meta.dwarf =
+        new DIFile(metadata.getOrNULL(metaRecord.ops[1]), metadata.getOrNULL(metaRecord.ops[2]));
+    meta.children = {metadata.getOrNULL(metaRecord.ops[1]), metadata.getOrNULL(metaRecord.ops[2])};
   }
   else if(id == LLVMBC::MetaDataRecord::COMPILE_UNIT)
   {
@@ -54,35 +61,39 @@ bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Meta
     meta.isDistinct = true;
 
     meta.dwarf = new DICompileUnit(
-        DW_LANG(metaRecord.ops[1]), getMeta(metaRecord.ops[2]), getMetaString(metaRecord.ops[3]),
-        metaRecord.ops[4] != 0, getMetaString(metaRecord.ops[5]), metaRecord.ops[6],
-        getMetaString(metaRecord.ops[7]), metaRecord.ops[8], getMeta(metaRecord.ops[9]),
-        getMeta(metaRecord.ops[10]), getMeta(metaRecord.ops[11]), getMeta(metaRecord.ops[12]),
-        getMeta(metaRecord.ops[13]));
-    meta.children = {getMeta(metaRecord.ops[2]),  getMeta(metaRecord.ops[9]),
-                     getMeta(metaRecord.ops[10]), getMeta(metaRecord.ops[11]),
-                     getMeta(metaRecord.ops[12]), getMeta(metaRecord.ops[13])};
+        DW_LANG(metaRecord.ops[1]), metadata.getOrNULL(metaRecord.ops[2]),
+        metadata.getStringOrNULL(metaRecord.ops[3]), metaRecord.ops[4] != 0,
+        metadata.getStringOrNULL(metaRecord.ops[5]), metaRecord.ops[6],
+        metadata.getStringOrNULL(metaRecord.ops[7]), metaRecord.ops[8],
+        metadata.getOrNULL(metaRecord.ops[9]), metadata.getOrNULL(metaRecord.ops[10]),
+        metadata.getOrNULL(metaRecord.ops[11]), metadata.getOrNULL(metaRecord.ops[12]),
+        metadata.getOrNULL(metaRecord.ops[13]));
+    meta.children = {
+        metadata.getOrNULL(metaRecord.ops[2]),  metadata.getOrNULL(metaRecord.ops[9]),
+        metadata.getOrNULL(metaRecord.ops[10]), metadata.getOrNULL(metaRecord.ops[11]),
+        metadata.getOrNULL(metaRecord.ops[12]), metadata.getOrNULL(metaRecord.ops[13])};
   }
   else if(id == LLVMBC::MetaDataRecord::BASIC_TYPE)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
     meta.dwarf =
-        new DIBasicType(DW_TAG(metaRecord.ops[1]), getMetaString(metaRecord.ops[2]),
+        new DIBasicType(DW_TAG(metaRecord.ops[1]), metadata.getStringOrNULL(metaRecord.ops[2]),
                         metaRecord.ops[3], metaRecord.ops[4], DW_ENCODING(metaRecord.ops[5]));
   }
   else if(id == LLVMBC::MetaDataRecord::DERIVED_TYPE)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
-    meta.dwarf = new DIDerivedType(DW_TAG(metaRecord.ops[1]), getMetaString(metaRecord.ops[2]),
-                                   getMeta(metaRecord.ops[3]), metaRecord.ops[4],
-                                   getMeta(metaRecord.ops[5]), getMeta(metaRecord.ops[6]),
-                                   metaRecord.ops[7], metaRecord.ops[8], metaRecord.ops[9],
-                                   DIFlags(metaRecord.ops[10]), getMeta(metaRecord.ops[11]));
+    meta.dwarf = new DIDerivedType(
+        DW_TAG(metaRecord.ops[1]), metadata.getStringOrNULL(metaRecord.ops[2]),
+        metadata.getOrNULL(metaRecord.ops[3]), metaRecord.ops[4],
+        metadata.getOrNULL(metaRecord.ops[5]), metadata.getOrNULL(metaRecord.ops[6]),
+        metaRecord.ops[7], metaRecord.ops[8], metaRecord.ops[9], DIFlags(metaRecord.ops[10]),
+        metadata.getOrNULL(metaRecord.ops[11]));
 
-    meta.children = {getMeta(metaRecord.ops[3]), getMeta(metaRecord.ops[5]),
-                     getMeta(metaRecord.ops[6]), getMeta(metaRecord.ops[11])};
+    meta.children = {metadata.getOrNULL(metaRecord.ops[3]), metadata.getOrNULL(metaRecord.ops[5]),
+                     metadata.getOrNULL(metaRecord.ops[6]), metadata.getOrNULL(metaRecord.ops[11])};
   }
   else if(id == LLVMBC::MetaDataRecord::COMPOSITE_TYPE)
   {
@@ -90,58 +101,69 @@ bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Meta
 
     // TODO handle forward declarations?
     meta.dwarf = new DICompositeType(
-        DW_TAG(metaRecord.ops[1]), getMetaString(metaRecord.ops[2]), getMeta(metaRecord.ops[3]),
-        metaRecord.ops[4], getMeta(metaRecord.ops[5]), getMeta(metaRecord.ops[6]),
+        DW_TAG(metaRecord.ops[1]), metadata.getStringOrNULL(metaRecord.ops[2]),
+        metadata.getOrNULL(metaRecord.ops[3]), metaRecord.ops[4],
+        metadata.getOrNULL(metaRecord.ops[5]), metadata.getOrNULL(metaRecord.ops[6]),
         metaRecord.ops[7], metaRecord.ops[8], metaRecord.ops[9], DIFlags(metaRecord.ops[10]),
-        getMeta(metaRecord.ops[11]), getMeta(metaRecord.ops[14]));
+        metadata.getOrNULL(metaRecord.ops[11]), metadata.getOrNULL(metaRecord.ops[14]));
 
-    meta.children = {getMeta(metaRecord.ops[3]), getMeta(metaRecord.ops[5]),
-                     getMeta(metaRecord.ops[6]), getMeta(metaRecord.ops[11]),
-                     getMeta(metaRecord.ops[14])};
+    meta.children = {metadata.getOrNULL(metaRecord.ops[3]), metadata.getOrNULL(metaRecord.ops[5]),
+                     metadata.getOrNULL(metaRecord.ops[6]), metadata.getOrNULL(metaRecord.ops[11]),
+                     metadata.getOrNULL(metaRecord.ops[14])};
+  }
+  else if(id == LLVMBC::MetaDataRecord::ENUMERATOR)
+  {
+    meta.isDistinct = (metaRecord.ops[0] & 0x1);
+
+    meta.dwarf =
+        new DIEnum(debug_only_svbr(metaRecord.ops[1]), metadata.getStringOrNULL(metaRecord.ops[2]));
   }
   else if(id == LLVMBC::MetaDataRecord::TEMPLATE_TYPE)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
-    meta.dwarf =
-        new DITemplateTypeParameter(getMetaString(metaRecord.ops[1]), getMeta(metaRecord.ops[2]));
+    meta.dwarf = new DITemplateTypeParameter(metadata.getStringOrNULL(metaRecord.ops[1]),
+                                             metadata.getOrNULL(metaRecord.ops[2]));
 
-    meta.children = {getMeta(metaRecord.ops[2])};
+    meta.children = {metadata.getOrNULL(metaRecord.ops[2])};
   }
   else if(id == LLVMBC::MetaDataRecord::TEMPLATE_VALUE)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
-    meta.dwarf =
-        new DITemplateValueParameter(DW_TAG(metaRecord.ops[1]), getMetaString(metaRecord.ops[2]),
-                                     getMeta(metaRecord.ops[3]), getMeta(metaRecord.ops[4]));
+    meta.dwarf = new DITemplateValueParameter(
+        DW_TAG(metaRecord.ops[1]), metadata.getStringOrNULL(metaRecord.ops[2]),
+        metadata.getOrNULL(metaRecord.ops[3]), metadata.getOrNULL(metaRecord.ops[4]));
 
-    meta.children = {getMeta(metaRecord.ops[3]), getMeta(metaRecord.ops[4])};
+    meta.children = {metadata.getOrNULL(metaRecord.ops[3]), metadata.getOrNULL(metaRecord.ops[4])};
   }
   else if(id == LLVMBC::MetaDataRecord::SUBPROGRAM)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
     meta.dwarf = new DISubprogram(
-        getMeta(metaRecord.ops[1]), getMetaString(metaRecord.ops[2]),
-        getMetaString(metaRecord.ops[3]), getMeta(metaRecord.ops[4]), metaRecord.ops[5],
-        getMeta(metaRecord.ops[6]), metaRecord.ops[7] != 0, metaRecord.ops[8] != 0, metaRecord.ops[9],
-        getMeta(metaRecord.ops[10]), DW_VIRTUALITY(metaRecord.ops[11]), metaRecord.ops[12],
-        DIFlags(metaRecord.ops[13]), metaRecord.ops[14] != 0, getMeta(metaRecord.ops[15]),
-        getMeta(metaRecord.ops[16]), getMeta(metaRecord.ops[17]), getMeta(metaRecord.ops[18]));
+        metadata.getOrNULL(metaRecord.ops[1]), metadata.getStringOrNULL(metaRecord.ops[2]),
+        metadata.getStringOrNULL(metaRecord.ops[3]), metadata.getOrNULL(metaRecord.ops[4]),
+        metaRecord.ops[5], metadata.getOrNULL(metaRecord.ops[6]), metaRecord.ops[7] != 0,
+        metaRecord.ops[8] != 0, metaRecord.ops[9], metadata.getOrNULL(metaRecord.ops[10]),
+        DW_VIRTUALITY(metaRecord.ops[11]), metaRecord.ops[12], DIFlags(metaRecord.ops[13]),
+        metaRecord.ops[14] != 0, metadata.getOrNULL(metaRecord.ops[15]),
+        metadata.getOrNULL(metaRecord.ops[16]), metadata.getOrNULL(metaRecord.ops[17]),
+        metadata.getOrNULL(metaRecord.ops[18]));
 
-    meta.children = {getMeta(metaRecord.ops[1]),  getMeta(metaRecord.ops[4]),
-                     getMeta(metaRecord.ops[6]),  getMeta(metaRecord.ops[10]),
-                     getMeta(metaRecord.ops[14]), getMeta(metaRecord.ops[15]),
-                     getMeta(metaRecord.ops[16]), getMeta(metaRecord.ops[17])};
+    meta.children = {
+        metadata.getOrNULL(metaRecord.ops[1]),  metadata.getOrNULL(metaRecord.ops[4]),
+        metadata.getOrNULL(metaRecord.ops[6]),  metadata.getOrNULL(metaRecord.ops[10]),
+        metadata.getOrNULL(metaRecord.ops[14]), metadata.getOrNULL(metaRecord.ops[15]),
+        metadata.getOrNULL(metaRecord.ops[16]), metadata.getOrNULL(metaRecord.ops[17])};
   }
   else if(id == LLVMBC::MetaDataRecord::SUBROUTINE_TYPE)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
-    meta.dwarf = new DISubroutineType(getMeta(metaRecord.ops[2]));
+    meta.dwarf = new DISubroutineType(metadata.getOrNULL(metaRecord.ops[2]));
 
-    meta.children = {getMeta(metaRecord.ops[2])};
+    meta.children = {metadata.getOrNULL(metaRecord.ops[2])};
   }
   else if(id == LLVMBC::MetaDataRecord::GLOBAL_VAR)
   {
@@ -152,14 +174,15 @@ bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Meta
     if(version == 0)
     {
       meta.dwarf = new DIGlobalVariable(
-          getMeta(metaRecord.ops[1]), getMetaString(metaRecord.ops[2]),
-          getMetaString(metaRecord.ops[3]), getMeta(metaRecord.ops[4]), metaRecord.ops[5],
-          getMeta(metaRecord.ops[6]), metaRecord.ops[7] != 0, metaRecord.ops[8] != 0,
-          getMeta(metaRecord.ops[9]), getMeta(metaRecord.ops[10]));
+          metadata.getOrNULL(metaRecord.ops[1]), metadata.getStringOrNULL(metaRecord.ops[2]),
+          metadata.getStringOrNULL(metaRecord.ops[3]), metadata.getOrNULL(metaRecord.ops[4]),
+          metaRecord.ops[5], metadata.getOrNULL(metaRecord.ops[6]), metaRecord.ops[7] != 0,
+          metaRecord.ops[8] != 0, metadata.getOrNULL(metaRecord.ops[9]),
+          metadata.getOrNULL(metaRecord.ops[10]));
 
-      meta.children = {getMeta(metaRecord.ops[1]), getMeta(metaRecord.ops[4]),
-                       getMeta(metaRecord.ops[6]), getMeta(metaRecord.ops[9]),
-                       getMeta(metaRecord.ops[10])};
+      meta.children = {metadata.getOrNULL(metaRecord.ops[1]), metadata.getOrNULL(metaRecord.ops[4]),
+                       metadata.getOrNULL(metaRecord.ops[6]), metadata.getOrNULL(metaRecord.ops[9]),
+                       metadata.getOrNULL(metaRecord.ops[10])};
     }
     else
     {
@@ -173,37 +196,60 @@ bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Meta
     meta.debugLoc = new DebugLocation;
     meta.debugLoc->line = metaRecord.ops[1];
     meta.debugLoc->col = metaRecord.ops[2];
-    meta.debugLoc->scope = getNonNullMeta(metaRecord.ops[3]);
-    meta.debugLoc->inlinedAt = getMeta(metaRecord.ops[4]);
+    meta.debugLoc->scope = metadata.getDirect(metaRecord.ops[3]);
+    meta.debugLoc->inlinedAt = metadata.getOrNULL(metaRecord.ops[4]);
 
-    meta.children = {getNonNullMeta(metaRecord.ops[3]), getMeta(metaRecord.ops[4])};
+    meta.children = {metadata.getDirect(metaRecord.ops[3]), metadata.getOrNULL(metaRecord.ops[4])};
   }
   else if(id == LLVMBC::MetaDataRecord::LOCAL_VAR)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
     meta.dwarf = new DILocalVariable(
-        DW_TAG(metaRecord.ops[1]), getMeta(metaRecord.ops[2]), getMetaString(metaRecord.ops[3]),
-        getMeta(metaRecord.ops[4]), metaRecord.ops[5], getMeta(metaRecord.ops[6]),
-        metaRecord.ops[7], DIFlags(metaRecord.ops[8]), metaRecord.ops[8]);
+        DW_TAG(metaRecord.ops[1]), metadata.getOrNULL(metaRecord.ops[2]),
+        metadata.getStringOrNULL(metaRecord.ops[3]), metadata.getOrNULL(metaRecord.ops[4]),
+        metaRecord.ops[5], metadata.getOrNULL(metaRecord.ops[6]), metaRecord.ops[7],
+        DIFlags(metaRecord.ops[8]));
 
-    meta.children = {getMeta(metaRecord.ops[2]), getMeta(metaRecord.ops[4]),
-                     getMeta(metaRecord.ops[6])};
+    meta.children = {metadata.getOrNULL(metaRecord.ops[2]), metadata.getOrNULL(metaRecord.ops[4]),
+                     metadata.getOrNULL(metaRecord.ops[6])};
   }
   else if(id == LLVMBC::MetaDataRecord::LEXICAL_BLOCK)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
-    meta.dwarf = new DILexicalBlock(getMeta(metaRecord.ops[1]), getMeta(metaRecord.ops[2]),
-                                    metaRecord.ops[3], metaRecord.ops[4]);
+    meta.dwarf = new DILexicalBlock(metadata.getOrNULL(metaRecord.ops[1]),
+                                    metadata.getOrNULL(metaRecord.ops[2]), metaRecord.ops[3],
+                                    metaRecord.ops[4]);
 
-    meta.children = {getMeta(metaRecord.ops[1]), getMeta(metaRecord.ops[2])};
+    meta.children = {metadata.getOrNULL(metaRecord.ops[1]), metadata.getOrNULL(metaRecord.ops[2])};
   }
   else if(id == LLVMBC::MetaDataRecord::SUBRANGE)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
-    meta.dwarf = new DISubrange(metaRecord.ops[1], LLVMBC::BitReader::svbr(metaRecord.ops[2]));
+    meta.dwarf = new DISubrange(int64_t(metaRecord.ops[1]), debug_only_svbr(metaRecord.ops[2]));
+  }
+  else if(id == LLVMBC::MetaDataRecord::NAMESPACE)
+  {
+    meta.isDistinct = (metaRecord.ops[0] & 0x1);
+
+    meta.dwarf = new DINamespace(metadata.getOrNULL(metaRecord.ops[1]),
+                                 metadata.getOrNULL(metaRecord.ops[2]),
+                                 metadata.getStringOrNULL(metaRecord.ops[3]), metaRecord.ops[4]);
+
+    meta.children = {metadata.getOrNULL(metaRecord.ops[1]), metadata.getOrNULL(metaRecord.ops[2])};
+  }
+  else if(id == LLVMBC::MetaDataRecord::IMPORTED_ENTITY)
+  {
+    meta.isDistinct = (metaRecord.ops[0] & 0x1);
+
+    meta.dwarf =
+        new DIImportedEntity(DW_TAG(metaRecord.ops[1]), metadata.getOrNULL(metaRecord.ops[2]),
+                             metadata.getOrNULL(metaRecord.ops[3]), metaRecord.ops[4],
+                             metadata.getStringOrNULL(metaRecord.ops[5]));
+
+    meta.children = {metadata.getOrNULL(metaRecord.ops[2]), metadata.getOrNULL(metaRecord.ops[3])};
   }
   else if(id == LLVMBC::MetaDataRecord::EXPRESSION)
   {
@@ -245,6 +291,33 @@ rdcstr Program::GetDebugVarName(const DIBase *d)
   return "???";
 }
 
+rdcstr Program::GetFunctionScopeName(const DIBase *d)
+{
+  const Metadata *scope = NULL;
+  if(d->type == DIBase::LocalVariable)
+    scope = d->As<DILocalVariable>()->scope;
+  if(d->type == DIBase::GlobalVariable)
+    scope = d->As<DIGlobalVariable>()->scope;
+
+  while(scope && scope->dwarf)
+  {
+    if(scope->dwarf->type == DIBase::Subprogram)
+    {
+      const rdcstr *name = scope->dwarf->As<DISubprogram>()->name;
+      return name ? *name : "";
+    }
+    else if(scope->dwarf->type == DIBase::LexicalBlock)
+    {
+      scope = scope->dwarf->As<DILexicalBlock>()->scope;
+      continue;
+    }
+
+    break;
+  }
+
+  return "";
+}
+
 rdcstr getOptMetaString(const Metadata *meta)
 {
   return meta ? escapeString(meta->str).c_str() : "\"\"";
@@ -357,6 +430,15 @@ rdcstr DICompositeType::toString() const
   return ret;
 }
 
+rdcstr DIEnum::toString() const
+{
+  rdcstr ret = "!DIEnumerator(";
+  ret += StringFormat::Fmt("name: %s", escapeString(*name).c_str());
+  ret += StringFormat::Fmt(", value: %lld", value);
+  ret += ")";
+  return ret;
+}
+
 rdcstr DITemplateTypeParameter::toString() const
 {
   return StringFormat::Fmt("!DITemplateTypeParameter(name: %s, type: %s)",
@@ -374,16 +456,17 @@ rdcstr DITemplateValueParameter::toString() const
 
 rdcstr DISubprogram::toString() const
 {
-  rdcstr ret =
-      StringFormat::Fmt("!DISubprogram(name: %s", escapeString(name ? *name : rdcstr()).c_str());
+  rdcstr ret = "!DISubprogram(";
+  if(name)
+    ret += StringFormat::Fmt("name: %s, ", escapeString(*name).c_str());
   if(linkageName)
-    ret += StringFormat::Fmt(", linkageName: %s", escapeString(*linkageName).c_str());
+    ret += StringFormat::Fmt("linkageName: %s, ", escapeString(*linkageName).c_str());
   if(scope)
-    ret += StringFormat::Fmt(", scope: %s", scope->refString().c_str());
+    ret += StringFormat::Fmt("scope: %s, ", scope->refString().c_str());
   if(file)
-    ret += StringFormat::Fmt(", file: %s", file->refString().c_str());
+    ret += StringFormat::Fmt("file: %s", file->refString().c_str());
   else
-    ret += ", file: null";
+    ret += "file: null";
   if(line)
     ret += StringFormat::Fmt(", line: %llu", line);
   if(type)
@@ -444,6 +527,8 @@ rdcstr DIGlobalVariable::toString() const
     ret += StringFormat::Fmt(", type: %s", type->refString().c_str());
   ret += StringFormat::Fmt(", isLocal: %s", isLocal ? "true" : "false");
   ret += StringFormat::Fmt(", isDefinition: %s", isDefinition ? "true" : "false");
+  if(declaration)
+    ret += StringFormat::Fmt(", declaration: %s", declaration->refString().c_str());
   if(variable)
     ret += StringFormat::Fmt(", variable: %s", variable->refString().c_str());
   ret += ")";
@@ -454,22 +539,20 @@ rdcstr DILocalVariable::toString() const
 {
   rdcstr ret = StringFormat::Fmt("!DILocalVariable(tag: %s, name: %s", ToStr(tag).c_str(),
                                  escapeString(name ? *name : rdcstr()).c_str());
-  if(arg)
+  if(arg || tag != DW_TAG_auto_variable)
     ret += StringFormat::Fmt(", arg: %llu", arg);
   if(scope)
     ret += StringFormat::Fmt(", scope: %s", scope->refString().c_str());
+  else
+    ret += ", scope: null";
   if(file)
     ret += StringFormat::Fmt(", file: %s", file->refString().c_str());
-  else
-    ret += ", file: null";
   if(line)
     ret += StringFormat::Fmt(", line: %llu", line);
   if(type)
     ret += StringFormat::Fmt(", type: %s", type->refString().c_str());
   if(flags != DIFlagNone)
     ret += StringFormat::Fmt(", flags: %s", ToStr(flags).c_str());
-  if(alignInBits)
-    ret += StringFormat::Fmt(", align: %llu", alignInBits);
   ret += ")";
   return ret;
 }
@@ -517,13 +600,46 @@ rdcstr DILexicalBlock::toString() const
 rdcstr DISubrange::toString() const
 {
   rdcstr ret = "!DISubrange(";
-  ret += StringFormat::Fmt("count: %llu", count);
+  ret += StringFormat::Fmt("count: %lld", count);
   if(lowerBound)
     ret += StringFormat::Fmt(", lowerBound: %lld", lowerBound);
   ret += ")";
   return ret;
 }
 
+rdcstr DINamespace::toString() const
+{
+  rdcstr ret = "!DINamespace(";
+  if(name)
+    ret += StringFormat::Fmt("name: %s, ", escapeString(*name).c_str());
+  if(scope)
+    ret += StringFormat::Fmt("scope: %s", scope->refString().c_str());
+  else
+    ret += "scope: null";
+  if(file)
+    ret += StringFormat::Fmt(", file: %s", file->refString().c_str());
+  ret += StringFormat::Fmt(", line: %llu", line);
+  ret += ")";
+  return ret;
+}
+
+rdcstr DIImportedEntity::toString() const
+{
+  rdcstr ret = StringFormat::Fmt("!DIImportedEntity(tag: %s", ToStr(tag).c_str());
+  if(name)
+    ret += StringFormat::Fmt(", name: %s, ", escapeString(*name).c_str());
+  if(scope)
+    ret += StringFormat::Fmt(", scope: %s", scope->refString().c_str());
+  else
+    ret += ", scope: null";
+  if(entity)
+    ret += StringFormat::Fmt(", entity: %s", entity->refString().c_str());
+  if(line)
+    ret += StringFormat::Fmt(", line: %llu", line);
+  ret += ")";
+  return ret;
+}
+
 };    // namespace DXIL
 
 template <>
