@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,6 +23,7 @@
  ******************************************************************************/
 
 #include "common/formatting.h"
+#include "core/settings.h"
 #include "maths/half_convert.h"
 #include "spirv_op_helpers.h"
 #include "spirv_reflect.h"
@@ -1574,6 +1575,48 @@ rdcstr Reflector::Disassemble(const rdcstr &entryPoint,
               ret += ToStr(rdcspv::SourceLanguage(lang));
               ret += ")";
             }
+            else if(dbg.inst == ShaderDbg::EntryPoint)
+            {
+              ret += indent;
+              ret += "DebugEntryPoint(";
+              OpShaderDbg debugFunc(GetID(dbg.arg<Id>(0)));
+              ret += idName(debugFunc.arg<Id>(0));
+              ret += ", ";
+              rdcstr gen = ToStr(m_Generator);
+              int i = gen.indexOf('-');
+              if(i > 0)
+                gen.erase(i - 1, ~0u);
+              ret += gen;
+              ret += " ";
+              ret += idName(dbg.arg<Id>(2));
+              rdcstr args = idName(dbg.arg<Id>(3));
+              if(!args.empty())
+              {
+                ret += ",\n";
+                lineNum++;
+                ret += indent + "    command line: ";
+                ret += args;
+              }
+
+              ret += ")";
+            }
+            else if(dbg.inst == ShaderDbg::Value)
+            {
+              OpShaderDbg localVar(GetID(dbg.arg<Id>(0)));
+              ret += indent;
+              ret += "// DebugValue(";
+              ret += idName(dbg.arg<Id>(1));
+              ret += " is ";
+              ret += idName(localVar.arg<Id>(0));
+              ret += " @ ";
+              ret += idName(localVar.arg<Id>(2));
+              ret += ":";
+              ret += idName(localVar.arg<Id>(3));
+
+              if(dbg.params.size() > 3)
+                ret += " (subset)";
+              ret += ")";
+            }
             else
             {
               continue;
@@ -1593,13 +1636,13 @@ rdcstr Reflector::Disassemble(const rdcstr &entryPoint,
 
             for(uint32_t i = 0; i < decoded.params.size(); i++)
             {
-              if(i == 5 && IsDebugPrintf)
+              if(i == 0 && IsDebugPrintf)
                 ret += "\"";
 
               // TODO could generate this from the instruction set grammar.
               ret += idParams ? idName(decoded.arg<Id>(i)) : ToStr(decoded.arg<uint32_t>(i));
 
-              if(i == 5 && IsDebugPrintf)
+              if(i == 0 && IsDebugPrintf)
                 ret += "\"";
 
               if(i + 1 < decoded.params.size())
