@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -47,6 +47,11 @@ Scalar::Scalar(Iter it)
     width = decoded.width;
     signedness = false;
   }
+  else if(type == Op::TypeBool)
+  {
+    width = 32;
+    signedness = false;
+  }
   else
   {
     width = 0;
