@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -678,6 +678,13 @@ struct VulkanCreationInfo
     bool customBorder;
     VkClearColorValue customBorderColor;
     VkFormat customBorderFormat;
+
+    // VkSamplerBorderColorComponentMappingCreateInfoEXT
+    VkComponentMapping componentMapping;
+    bool srgbBorder;
+
+    // VK_SAMPLER_CREATE_NON_SEAMLESS_CUBE_MAP_BIT_EXT
+    bool seamless;
   };
   std::unordered_map<ResourceId, Sampler> m_Sampler;
 
@@ -706,6 +713,9 @@ struct VulkanCreationInfo
     VkFormat format;
     VkImageSubresourceRange range;
     VkComponentMapping componentMapping;
+
+    // VkImageViewMinLodCreateInfoEXT
+    float minLOD;
   };
   std::unordered_map<ResourceId, ImageView> m_ImageView;
 
@@ -718,6 +728,10 @@ struct VulkanCreationInfo
 
     ShaderModuleReflection &GetReflection(ShaderStage stage, const rdcstr &entry, ResourceId pipe)
     {
+      auto redirIt = m_PipeReferences.find(pipe);
+      if(redirIt != m_PipeReferences.end())
+        pipe = redirIt->second;
+
       // look for one from this pipeline specifically, if it was specialised
       auto it = m_Reflections.find({stage, entry, pipe});
       if(it != m_Reflections.end())
@@ -732,6 +746,10 @@ struct VulkanCreationInfo
     rdcstr unstrippedPath;
 
     std::map<ShaderModuleReflectionKey, ShaderModuleReflection> m_Reflections;
+    // in graphics pipeline library the linked pipeline may reference a different pipeline where the
+    // shaders are. So when looking up the reflection as specialised by a given pipeline we may want
+    // to redirect to the 'real' pipeline that specialised it.
+    std::unordered_map<ResourceId, ResourceId> m_PipeReferences;
   };
   std::unordered_map<ResourceId, ShaderModule> m_ShaderModule;
 
