@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -207,6 +207,10 @@ MemoryAllocation WrappedVulkan::AllocateMemoryForResource(bool buffer, VkMemoryR
     // offs is where we can put our next sub-allocation
     VkDeviceSize offs = block.offs;
 
+    // for ease, ensure all allocations are allocated to the non-coherent atom size, so we can
+    // invalidate/flush safely. This is at most 256 bytes which is likely already satisfied.
+    offs = AlignUp(offs, nonCoherentAtomSize);
+
     // if we are on a buffer/image, account for any alignment we might have to do
     if(ret.buffer != block.buffer)
       offs = AlignUp(offs, m_PhysicalDeviceData.props.limits.bufferImageGranularity);
@@ -404,15 +408,39 @@ void WrappedVulkan::FreeAllMemory(MemoryScope scope)
   if(allocList.empty())
     return;
 
-  VkDevice d = GetDev();
-
-  for(MemoryAllocation alloc : allocList)
+  // freeing a lot of memory can take a while on some implementations. Since this only needs to
+  // externally synchronise the memory we do it on a thread and synchronise if we need to free again
+  // or on device shutdown
+  if(m_MemoryFreeThread)
   {
-    ObjDisp(d)->FreeMemory(Unwrap(d), Unwrap(alloc.mem), NULL);
-    GetResourceManager()->ReleaseWrappedResource(alloc.mem);
+    Threading::JoinThread(m_MemoryFreeThread);
+    Threading::CloseThread(m_MemoryFreeThread);
+    m_MemoryFreeThread = 0;
   }
 
-  allocList.clear();
+  VkDevice d = GetDev();
+
+  rdcarray<MemoryAllocation> allocs;
+  allocs.swap(allocList);
+
+  m_MemoryFreeThread = Threading::CreateThread([this, d, allocs]() {
+    for(const MemoryAllocation &alloc : allocs)
+    {
+      ObjDisp(d)->FreeMemory(Unwrap(d), Unwrap(alloc.mem), NULL);
+      GetResourceManager()->ReleaseWrappedResource(alloc.mem);
+    }
+  });
+}
+
+void WrappedVulkan::ResetMemoryBlocks(MemoryScope scope)
+{
+  rdcarray<MemoryAllocation> &allocList = m_MemoryBlocks[(size_t)scope];
+
+  if(allocList.empty())
+    return;
+
+  for(MemoryAllocation &alloc : allocList)
+    alloc.offs = 0;
 }
 
 void WrappedVulkan::FreeMemoryAllocation(MemoryAllocation alloc)
