@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -365,17 +365,22 @@ rdcstr LayerRegistrationPath(LayerPath path)
 {
   switch(path)
   {
-    case LayerPath::usr: return "/usr/share/vulkan/implicit_layer.d/renderdoc_capture.json";
-    case LayerPath::etc: return "/etc/vulkan/implicit_layer.d/renderdoc_capture.json";
+    case LayerPath::usr:
+      return "/usr/share/vulkan/implicit_layer.d/renderdoc_capture" STRINGIZE(
+          RENDERDOC_VULKAN_JSON_SUFFIX) ".json";
+    case LayerPath::etc:
+      return "/etc/vulkan/implicit_layer.d/renderdoc_capture" STRINGIZE(
+          RENDERDOC_VULKAN_JSON_SUFFIX) ".json";
     case LayerPath::home:
     {
-      const char *xdg = getenv("XDG_DATA_HOME");
-      if(xdg && FileIO::exists(xdg))
-        return rdcstr(xdg) + "/vulkan/implicit_layer.d/renderdoc_capture.json";
+      rdcstr xdg = Process::GetEnvVariable("XDG_DATA_HOME");
+      if(!xdg.empty() && FileIO::exists(xdg))
+        return xdg + "/vulkan/implicit_layer.d/renderdoc_capture" STRINGIZE(
+                         RENDERDOC_VULKAN_JSON_SUFFIX) ".json";
 
-      const char *home_path = getenv("HOME");
-      return rdcstr(home_path != NULL ? home_path : "") +
-             "/.local/share/vulkan/implicit_layer.d/renderdoc_capture.json";
+      rdcstr home_path = Process::GetEnvVariable("HOME");
+      return home_path + "/.local/share/vulkan/implicit_layer.d/renderdoc_capture" STRINGIZE(
+                             RENDERDOC_VULKAN_JSON_SUFFIX) ".json";
     }
     default: break;
   }
