@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -152,6 +152,8 @@ class VulkanResourceManager;
 struct VulkanStatePipeline;
 struct VulkanAMDActionCallback;
 
+class NVVulkanCounters;
+
 struct VulkanPostVSData
 {
   struct InstData
@@ -439,12 +441,15 @@ public:
                                rdcarray<rdcstr> &otherJSONs);
   static void InstallVulkanLayer(bool systemLevel);
   void GetInitialDriverVersion();
-  void SetDriverInformation(const VkPhysicalDeviceProperties &props);
+  void SetDriverInformation(const VkPhysicalDeviceProperties &props,
+                            const VkPhysicalDeviceDriverProperties &driverProps);
 
   AMDCounters *GetAMDCounters() { return m_pAMDCounters; }
   void CopyPixelForPixelHistory(VkCommandBuffer cmd, VkOffset2D offset, uint32_t sample,
                                 uint32_t bufferOffset, VkFormat format, VkDescriptorSet descSet);
 
+  bool Depth3DSupported() { return m_TexRender.DummyImages[3][2] != VK_NULL_HANDLE; }
+  bool DepthCubeSupported() { return m_TexRender.DepthCubesSupported; }
 private:
   bool FetchShaderFeedback(uint32_t eventId);
   void ClearFeedbackCache();
@@ -602,6 +607,7 @@ private:
     VkSampler DummySampler = VK_NULL_HANDLE;
     VkBuffer DummyBuffer = VK_NULL_HANDLE;
     VkBufferView DummyBufferView[4] = {};
+    bool DepthCubesSupported = true;
 
     std::map<ResourceId, TextureDisplayViews> TextureViews;
 
@@ -792,6 +798,10 @@ private:
 
   VulkanAMDActionCallback *m_pAMDActionCallback = NULL;
 
+#if DISABLED(RDOC_ANDROID) && DISABLED(RDOC_APPLE)
+  NVVulkanCounters *m_pNVCounters = NULL;
+#endif
+
   rdcarray<CounterResult> FetchCountersKHR(const rdcarray<GPUCounter> &counters);
 
   rdcarray<VkPerformanceCounterKHR> m_KHRCounters;
