@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -2161,7 +2161,7 @@ struct VkResourceRecord : public ResourceRecord
 public:
   enum
   {
-    NullResource = VK_NULL_HANDLE
+    NullResource = 0u
   };
 
   static byte markerValue[32];
