@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -229,7 +229,7 @@ VulkanShaderCache::VulkanShaderCache(WrappedVulkan *driver)
 
   SetCaching(true);
 
-  VkDriverInfo driverVersion = driver->GetDriverInfo();
+  const VkDriverInfo &driverVersion = driver->GetDriverInfo();
   const VkPhysicalDeviceFeatures &enabledFeatures = driver->GetDeviceEnabledFeatures();
   const VkPhysicalDeviceFeatures &availFeatures = driver->GetDeviceAvailableFeatures();
 
@@ -952,7 +952,7 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
   if(pipeInfo.renderpass == ResourceId())
   {
     dynRenderCreate.depthAttachmentFormat = pipeInfo.depthFormat;
-    dynRenderCreate.stencilAttachmentFormat = pipeInfo.depthFormat;
+    dynRenderCreate.stencilAttachmentFormat = pipeInfo.stencilFormat;
     dynRenderCreate.colorAttachmentCount = (uint32_t)pipeInfo.colorFormats.size();
     memcpy(colFormats, pipeInfo.colorFormats.data(), pipeInfo.colorFormats.byteSize());
 
