@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -96,7 +96,7 @@ void setupRenderingInfo(const VulkanRenderState::DynamicRendering &dynamicRender
 
   structs->fragmentDensity = {
       VK_STRUCTURE_TYPE_RENDERING_FRAGMENT_DENSITY_MAP_ATTACHMENT_INFO_EXT, NULL,
-      dynamicRendering.fragmentDensityView, dynamicRendering.fragmentDensityLayout,
+      Unwrap(dynamicRendering.fragmentDensityView), dynamicRendering.fragmentDensityLayout,
   };
 
   if(dynamicRendering.fragmentDensityView != VK_NULL_HANDLE)
@@ -108,7 +108,7 @@ void setupRenderingInfo(const VulkanRenderState::DynamicRendering &dynamicRender
   structs->shadingRate = {
       VK_STRUCTURE_TYPE_RENDERING_FRAGMENT_SHADING_RATE_ATTACHMENT_INFO_KHR,
       NULL,
-      dynamicRendering.shadingRateView,
+      Unwrap(dynamicRendering.shadingRateView),
       dynamicRendering.shadingRateLayout,
       dynamicRendering.shadingRateTexelSize,
   };
@@ -893,6 +893,9 @@ void VulkanRenderState::BindDescriptorSet(WrappedVulkan *vk, const DescSetLayout
           dst[a].imageLayout = convert(slots[a].imageLayout);
           dst[a].sampler = Unwrap(rm->GetCurrentHandle<VkSampler>(slots[a].sampler));
           dst[a].imageView = Unwrap(rm->GetCurrentHandle<VkImageView>(slots[a].resource));
+
+          if(layoutBind.immutableSampler && push.descriptorType != VK_DESCRIPTOR_TYPE_SAMPLER)
+            dst[a].sampler = Unwrap(rm->GetCurrentHandle<VkSampler>(layoutBind.immutableSampler[a]));
         }
 
         push.pImageInfo = dst;
