@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -85,6 +85,11 @@ rdcstr GetAppFolderFilename(const rdcstr &filename)
   return GetTempRootPath() + rdcstr("/") + filename;
 }
 
+rdcstr FindFileInPath(const rdcstr &fileName)
+{
+  return fileName;
+}
+
 // For RenderDoc's apk, this returns our package name
 // For other APKs, we use it to get the writable temp directory.
 void GetExecutableFilename(rdcstr &selfName)
