@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -67,8 +67,8 @@ rdcstr apple_GetExecutablePathFromAppBundle(const char *appBundlePath)
   NSBundle *nsBundle = [NSBundle bundleWithPath:path];
   if(!nsBundle)
   {
-    RDCERR("Failed to open application '%s' as an NSBundle", appBundlePath);
-    return rdcstr();
+    RDCWARN("Failed to open application '%s' as an NSBundle", appBundlePath);
+    return rdcstr(appBundlePath);
   }
 
   NSString *executablePath = nsBundle.executablePath;
