@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -742,7 +742,7 @@ ResultDetails ReplayController::SaveTexture(const TextureSave &saveData, const r
   // for non-HDR always downcast if we're not already RGBA8 unorm
   if(sd.destType != FileType::DDS && sd.destType != FileType::HDR && sd.destType != FileType::EXR &&
      (td.format.compByteWidth != 1 || td.format.compCount != 4 ||
-      td.format.compType != CompType::UNorm || td.format.BGRAOrder()))
+      td.format.compType != CompType::UNorm || td.format.BGRAOrder() || td.format.Special()))
     downcast = true;
 
   // for HDR & EXR we can convert from most regular types as well as 10.10.10.2 and 11.11.10
