@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -48,6 +48,8 @@ public:
   void ClearThumbnails();
   ResultDetails AddThumbnail(WindowingData window, ResourceId texID, const Subresource &sub,
                              CompType typeCast);
+  bytebuf DrawThumbnail(int32_t width, int32_t height, ResourceId textureId, const Subresource &sub,
+                        CompType typeCast);
 
   void Display();
 
@@ -97,10 +99,16 @@ private:
     bool dirty;
   } m_MainOutput;
 
+  rdcpair<uint32_t, uint32_t> m_TextureDim = {0, 0};
+
   ResourceId m_OverlayResourceId;
   ResourceId m_CustomShaderResourceId;
 
   rdcarray<OutputPair> m_Thumbnails;
+  rdcarray<rdcpair<uint64_t, uint64_t>> m_ThumbnailGenerators;
+  // keep 8 generators to avoid churn, but most thumbnails should be the same size so this means
+  // during resize we don't create and destroy too many
+  static const size_t MaxThumbnailGenerators = 8;
 
   float m_ContextX;
   float m_ContextY;
