@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -101,7 +101,11 @@ struct GetTextureDataParams
 
 DECLARE_REFLECTION_STRUCT(GetTextureDataParams);
 
-CompType BaseRemapType(CompType typeCast);
+CompType BaseRemapType(RemapTexture remap, CompType typeCast);
+inline CompType BaseRemapType(const GetTextureDataParams &params)
+{
+  return BaseRemapType(params.remap, params.typeCast);
+}
 
 class RDCFile;
 
