@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -379,6 +379,9 @@ bytebuf ReplayOutput::DrawThumbnail(int32_t width, int32_t height, ResourceId te
 {
   bytebuf ret;
 
+  width = RDCMAX(width, 1);
+  height = RDCMAX(height, 1);
+
   uint64_t key = (uint64_t(width) << 32) | height;
   int idx = -1;
   uint64_t outputID = 0;
@@ -729,11 +732,11 @@ void ReplayOutput::DisplayContext()
     rdcpair<uint32_t, uint32_t> mipDim = {RDCMAX(1U, m_TextureDim.first >> disp.subresource.mip),
                                           RDCMAX(1U, m_TextureDim.second >> disp.subresource.mip)};
 
-    x = int((float(x) / float(m_TextureDim.first)) * mipDim.first);
-    x = int((float(x) / float(mipDim.first)) * m_TextureDim.first);
+    x = int((float(x) / float(m_TextureDim.first) + 1e-6f) * mipDim.first);
+    x = int((float(x) / float(mipDim.first) + 1e-6f) * m_TextureDim.first);
 
-    y = int((float(y) / float(m_TextureDim.second)) * mipDim.second);
-    y = int((float(y) / float(mipDim.second)) * m_TextureDim.second);
+    y = int((float(y) / float(m_TextureDim.second) + 1e-6f) * mipDim.second);
+    y = int((float(y) / float(mipDim.second) + 1e-6f) * m_TextureDim.second);
   }
   else
   {
