@@ -3,7 +3,7 @@
 /* this ALWAYS GENERATED file contains the definitions for the interfaces */
 
 
- /* File created by MIDL compiler version 8.01.0618 */
+ /* File created by MIDL compiler version 8.01.0622 */
 /* @@MIDL_FILE_HEADING(  ) */
 
 
@@ -140,6 +140,7 @@ typedef interface IDXGIDevice1 IDXGIDevice1;
 /* header files for imported files */
 #include "oaidl.h"
 #include "ocidl.h"
+#include "dxgicommon.h"
 #include "dxgitype.h"
 
 #ifdef __cplusplus
@@ -181,6 +182,8 @@ typedef struct DXGI_MAPPED_RECT
     } 	DXGI_MAPPED_RECT;
 
 #ifdef __midl
+#ifndef LUID_DEFINED
+#define LUID_DEFINED 1
 typedef struct _LUID
     {
     DWORD LowPart;
@@ -189,6 +192,7 @@ typedef struct _LUID
 
 typedef struct _LUID *PLUID;
 
+#endif
 #endif
 typedef struct DXGI_ADAPTER_DESC
     {
@@ -274,7 +278,8 @@ enum DXGI_SWAP_CHAIN_FLAG
         DXGI_SWAP_CHAIN_FLAG_FULLSCREEN_VIDEO	= 256,
         DXGI_SWAP_CHAIN_FLAG_YUV_VIDEO	= 512,
         DXGI_SWAP_CHAIN_FLAG_HW_PROTECTED	= 1024,
-        DXGI_SWAP_CHAIN_FLAG_ALLOW_TEARING	= 2048
+        DXGI_SWAP_CHAIN_FLAG_ALLOW_TEARING	= 2048,
+        DXGI_SWAP_CHAIN_FLAG_RESTRICTED_TO_ALL_HOLOGRAPHIC_DISPLAYS	= 4096
     } 	DXGI_SWAP_CHAIN_FLAG;
 
 typedef struct DXGI_SWAP_CHAIN_DESC
@@ -320,7 +325,7 @@ EXTERN_C const IID IID_IDXGIObject;
             /* [annotation][in] */ 
             _In_  REFGUID Name,
             /* [annotation][in] */ 
-            _In_  const IUnknown *pUnknown) = 0;
+            _In_opt_  const IUnknown *pUnknown) = 0;
         
         virtual HRESULT STDMETHODCALLTYPE GetPrivateData( 
             /* [annotation][in] */ 
@@ -370,7 +375,7 @@ EXTERN_C const IID IID_IDXGIObject;
             /* [annotation][in] */ 
             _In_  REFGUID Name,
             /* [annotation][in] */ 
-            _In_  const IUnknown *pUnknown);
+            _In_opt_  const IUnknown *pUnknown);
         
         HRESULT ( STDMETHODCALLTYPE *GetPrivateData )( 
             IDXGIObject * This,
@@ -489,7 +494,7 @@ EXTERN_C const IID IID_IDXGIDeviceSubObject;
             /* [annotation][in] */ 
             _In_  REFGUID Name,
             /* [annotation][in] */ 
-            _In_  const IUnknown *pUnknown);
+            _In_opt_  const IUnknown *pUnknown);
         
         HRESULT ( STDMETHODCALLTYPE *GetPrivateData )( 
             IDXGIDeviceSubObject * This,
@@ -627,7 +632,7 @@ EXTERN_C const IID IID_IDXGIResource;
             /* [annotation][in] */ 
             _In_  REFGUID Name,
             /* [annotation][in] */ 
-            _In_  const IUnknown *pUnknown);
+            _In_opt_  const IUnknown *pUnknown);
         
         HRESULT ( STDMETHODCALLTYPE *GetPrivateData )( 
             IDXGIResource * This,
@@ -789,7 +794,7 @@ EXTERN_C const IID IID_IDXGIKeyedMutex;
             /* [annotation][in] */ 
             _In_  REFGUID Name,
             /* [annotation][in] */ 
-            _In_  const IUnknown *pUnknown);
+            _In_opt_  const IUnknown *pUnknown);
         
         HRESULT ( STDMETHODCALLTYPE *GetPrivateData )( 
             IDXGIKeyedMutex * This,
@@ -954,7 +959,7 @@ EXTERN_C const IID IID_IDXGISurface;
             /* [annotation][in] */ 
             _In_  REFGUID Name,
             /* [annotation][in] */ 
-            _In_  const IUnknown *pUnknown);
+            _In_opt_  const IUnknown *pUnknown);
         
         HRESULT ( STDMETHODCALLTYPE *GetPrivateData )( 
             IDXGISurface * This,
@@ -1111,7 +1116,7 @@ EXTERN_C const IID IID_IDXGISurface1;
             /* [annotation][in] */ 
             _In_  REFGUID Name,
             /* [annotation][in] */ 
-            _In_  const IUnknown *pUnknown);
+            _In_opt_  const IUnknown *pUnknown);
         
         HRESULT ( STDMETHODCALLTYPE *GetPrivateData )( 
             IDXGISurface1 * This,
@@ -1301,7 +1306,7 @@ EXTERN_C const IID IID_IDXGIAdapter;
             /* [annotation][in] */ 
             _In_  REFGUID Name,
             /* [annotation][in] */ 
-            _In_  const IUnknown *pUnknown);
+            _In_opt_  const IUnknown *pUnknown);
         
         HRESULT ( STDMETHODCALLTYPE *GetPrivateData )( 
             IDXGIAdapter * This,
@@ -1507,7 +1512,7 @@ EXTERN_C const IID IID_IDXGIOutput;
             /* [annotation][in] */ 
             _In_  REFGUID Name,
             /* [annotation][in] */ 
-            _In_  const IUnknown *pUnknown);
+            _In_opt_  const IUnknown *pUnknown);
         
         HRESULT ( STDMETHODCALLTYPE *GetPrivateData )( 
             IDXGIOutput * This,
@@ -1789,7 +1794,7 @@ EXTERN_C const IID IID_IDXGISwapChain;
             /* [annotation][in] */ 
             _In_  REFGUID Name,
             /* [annotation][in] */ 
-            _In_  const IUnknown *pUnknown);
+            _In_opt_  const IUnknown *pUnknown);
         
         HRESULT ( STDMETHODCALLTYPE *GetPrivateData )( 
             IDXGISwapChain * This,
@@ -2041,7 +2046,7 @@ EXTERN_C const IID IID_IDXGIFactory;
             /* [annotation][in] */ 
             _In_  REFGUID Name,
             /* [annotation][in] */ 
-            _In_  const IUnknown *pUnknown);
+            _In_opt_  const IUnknown *pUnknown);
         
         HRESULT ( STDMETHODCALLTYPE *GetPrivateData )( 
             IDXGIFactory * This,
@@ -2243,7 +2248,7 @@ EXTERN_C const IID IID_IDXGIDevice;
             /* [annotation][in] */ 
             _In_  REFGUID Name,
             /* [annotation][in] */ 
-            _In_  const IUnknown *pUnknown);
+            _In_opt_  const IUnknown *pUnknown);
         
         HRESULT ( STDMETHODCALLTYPE *GetPrivateData )( 
             IDXGIDevice * This,
@@ -2450,7 +2455,7 @@ EXTERN_C const IID IID_IDXGIFactory1;
             /* [annotation][in] */ 
             _In_  REFGUID Name,
             /* [annotation][in] */ 
-            _In_  const IUnknown *pUnknown);
+            _In_opt_  const IUnknown *pUnknown);
         
         HRESULT ( STDMETHODCALLTYPE *GetPrivateData )( 
             IDXGIFactory1 * This,
@@ -2630,7 +2635,7 @@ EXTERN_C const IID IID_IDXGIAdapter1;
             /* [annotation][in] */ 
             _In_  REFGUID Name,
             /* [annotation][in] */ 
-            _In_  const IUnknown *pUnknown);
+            _In_opt_  const IUnknown *pUnknown);
         
         HRESULT ( STDMETHODCALLTYPE *GetPrivateData )( 
             IDXGIAdapter1 * This,
@@ -2787,7 +2792,7 @@ EXTERN_C const IID IID_IDXGIDevice1;
             /* [annotation][in] */ 
             _In_  REFGUID Name,
             /* [annotation][in] */ 
-            _In_  const IUnknown *pUnknown);
+            _In_opt_  const IUnknown *pUnknown);
         
         HRESULT ( STDMETHODCALLTYPE *GetPrivateData )( 
             IDXGIDevice1 * This,
