@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -294,6 +294,19 @@ void main()
           glDrawArrays(GL_TRIANGLES, 33, 3);
         }
 
+        if(fb == msaafbo)
+        {
+          setMarker("Sample Mask Test");
+          glDisable(GL_STENCIL_TEST);
+          glEnable(GL_SAMPLE_MASK);
+          glSampleMaski(0, 0x2);
+          glViewport(0, screenHeight - 80, 80, 80);
+          glScissor(0, screenHeight - 80, 80, 80);
+          glDrawArrays(GL_TRIANGLES, 6, 3);
+          glSampleMaski(0, ~0U);
+          glDisable(GL_SAMPLE_MASK);
+        }
+
         glScissor(0, 0, screenWidth, screenHeight);
       }
 
