@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2022 Baldur Karlsson
+ * Copyright (c) 2019-2023 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -401,7 +401,8 @@ void main()
     Vec4i dimensions(texWidth, texHeight, texDepth);
 
     bool isCompressed =
-        (test.fmt.cfg.type != TextureType::R9G9B9E5 && test.fmt.cfg.type != TextureType::Regular);
+        (test.fmt.cfg.type != TextureType::R9G9B9E5 && test.fmt.cfg.type != TextureType::Regular) ||
+        test.fmt.internalFormat == GL_STENCIL_INDEX8;
 
     // Some GL drivers report that block compressed textures are supported for MSAA and color
     // rendering. Save them from themselves. Similarly they report support for 1D and 3D but then it
@@ -590,6 +591,9 @@ void main()
     else if(test.fmt.cfg.componentCount == 1)
       format = isInt ? GL_RED_INTEGER : GL_RED;
 
+    if(test.fmt.internalFormat == GL_STENCIL_INDEX8)
+      format = GL_STENCIL_INDEX;
+
     if(test.fmt.cfg.type == TextureType::R9G9B9E5)
     {
       format = GL_RGB;
@@ -881,6 +885,8 @@ void main()
         TEST_CASE(TextureType::Unknown, GL_DEPTH_COMPONENT24, 0, 0, DataType::Float),
 
         TEST_CASE(TextureType::Unknown, GL_DEPTH_COMPONENT16, 0, 0, DataType::Float),
+
+        TEST_CASE(TextureType::Regular, GL_STENCIL_INDEX8, 1, 1, DataType::UInt),
     };
 
     for(GLFormat f : depth_tests)
