@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2019-2022 Baldur Karlsson
+* Copyright (c) 2019-2023 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -75,7 +75,7 @@ void MakePixel(byte *data, const TexConfig &cfg, uint32_t x, uint32_t y, uint32_
         else if(cfg.data == DataType::UNorm)
           h = uint16_t(f * 0xffff);
         else if(cfg.data == DataType::SNorm)
-          h = int16_t(f * 0x7fff);
+          h = f < 0 ? int16_t(roundf(f * 0x8000)) : int16_t(roundf(f * 0x7fff));
         memcpy(data, &h, cfg.componentBytes);
       }
       else if(cfg.componentBytes == 1)
@@ -84,7 +84,7 @@ void MakePixel(byte *data, const TexConfig &cfg, uint32_t x, uint32_t y, uint32_
         if(cfg.data == DataType::UNorm)
           b = uint8_t(f * 0xff);
         else if(cfg.data == DataType::SNorm)
-          b = int8_t(f * 0x7f);
+          b = f < 0 ? int8_t(roundf(f * 0x80)) : int8_t(roundf(f * 0x7f));
         memcpy(data, &b, cfg.componentBytes);
       }
       else
