@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2019-2022 Baldur Karlsson
+* Copyright (c) 2019-2023 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -720,11 +720,12 @@ struct PushConstantRange : public VkPushConstantRange
 struct PipelineLayoutCreateInfo : public VkPipelineLayoutCreateInfo
 {
   PipelineLayoutCreateInfo(const std::vector<VkDescriptorSetLayout> &setLayouts = {},
-                           const std::vector<VkPushConstantRange> &pushConstantRanges = {})
+                           const std::vector<VkPushConstantRange> &pushConstantRanges = {},
+                           VkPipelineLayoutCreateFlags flags = 0)
   {
     sType = VK_STRUCTURE_TYPE_PIPELINE_LAYOUT_CREATE_INFO;
     pNext = NULL;
-    this->flags = 0;
+    this->flags = flags;
     this->setLayoutCount = (uint32_t)setLayouts.size();
     this->pSetLayouts = setLayouts.data();
     this->pushConstantRangeCount = (uint32_t)pushConstantRanges.size();
@@ -1369,6 +1370,12 @@ struct GraphicsPipelineCreateInfo : private VkGraphicsPipelineCreateInfo
     return (const VkGraphicsPipelineCreateInfo *)this;
   }
 
+  operator VkGraphicsPipelineCreateInfo *()
+  {
+    bake();
+    return (VkGraphicsPipelineCreateInfo *)this;
+  }
+
 private:
   void bake();
 };
