@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2019-2022 Baldur Karlsson
+* Copyright (c) 2019-2023 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -271,6 +271,13 @@ struct VulkanGraphicsTest : public GraphicsTest
   VkQueueFlags queueFlagsRequired = VK_QUEUE_GRAPHICS_BIT | VK_QUEUE_COMPUTE_BIT;
   VkQueueFlags queueFlagsBanned = 0;
 
+  bool forceGraphicsQueue = false;
+  bool forceComputeQueue = false;
+  bool forceTransferQueue = false;
+  uint32_t graphicsQueueFamilyIndex = ~0U;
+  uint32_t computeQueueFamilyIndex = ~0U;
+  uint32_t transferQueueFamilyIndex = ~0U;
+
   bool hasExt(const char *ext);
 
   // a custom struct to pass to vkDeviceCreateInfo::pNext
