@@ -1,8 +1,5 @@
 import renderdoc as rd
 import rdtest
-from typing import List, Tuple
-import time
-import os
 
 # Not a real test, re-used by API-specific tests
 class Mesh_Zoo():
@@ -23,18 +20,18 @@ class Mesh_Zoo():
 
         rdtest.png_save(rdtest.get_tmp_path('output.png'), self.rows, dim, False)
 
-    def find_draw(self, name):
-        draw = None
+    def find_action(self, name):
+        action = None
 
-        for d in self.controller.GetDrawcalls():
-            if name in d.name:
-                draw = d
+        for d in self.controller.GetRootActions():
+            if name in d.customName:
+                action = d
                 break
 
-        if draw is None:
-            raise rdtest.TestFailureException("Couldn't find '{}' draw".format(name))
+        if action is None:
+            raise rdtest.TestFailureException("Couldn't find '{}' action".format(name))
 
-        return draw
+        return action
 
     # To avoid needing to do image comparisons, we instead do quad region probes to see which colours are present. That
     # way we can programmatically check that the wireframe we expect to be there, is there
@@ -72,7 +69,7 @@ class Mesh_Zoo():
     def check_capture(self, capture_filename: str, controller: rd.ReplayController):
         self.controller = controller
 
-        self.controller.SetFrameEvent(self.find_draw("Quad").next.eventId, False)
+        self.controller.SetFrameEvent(self.find_action("Quad").next.eventId, False)
 
         self.out: rd.ReplayOutput = self.controller.CreateOutput(rd.CreateHeadlessWindowingData(200, 200),
                                                             rd.ReplayOutputType.Mesh)
@@ -280,7 +277,7 @@ class Mesh_Zoo():
         self.check_region((125, 85, 135, 85), lambda x: x != [])
         self.check_region((105, 110, 105, 120), lambda x: x != [])
 
-        # But now an extra previous draw
+        # But now an extra previous action
         self.check_region((30, 105, 40, 105), lambda x: x != [])
         self.check_region((50, 80, 50, 90), lambda x: x != [])
         self.check_region((45, 130, 55, 130), lambda x: x != [])
@@ -351,7 +348,7 @@ class Mesh_Zoo():
 
         rdtest.log.success("Both instance picking is as expected")
 
-        self.controller.SetFrameEvent(self.find_draw("Points").next.eventId, False)
+        self.controller.SetFrameEvent(self.find_action("Points").next.eventId, False)
 
         # Only one instance, just check we can see the points
         self.cfg.curInstance = 0
@@ -372,7 +369,24 @@ class Mesh_Zoo():
 
         rdtest.log.success("Point picking is as expected")
 
-        self.controller.SetFrameEvent(self.find_draw("Stride 0").next.eventId, False)
+        self.cfg.highlightVert = rd.MeshDisplay.NoHighlight
+        self.cfg.solidShadeMode = rd.SolidShade.Solid
+
+        self.cache_output()
+        self.cfg.solidShadeMode = rd.SolidShade.Lit
+        self.cache_output()
+
+        rdtest.log.success("Point solid and lit rendering works as expected")
+
+        self.controller.SetFrameEvent(self.find_action("Lines").next.eventId, False)
+
+        self.cache_output()
+        self.cfg.solidShadeMode = rd.SolidShade.Lit
+        self.cache_output()
+
+        rdtest.log.success("Lines solid and lit rendering works as expected")
+
+        self.controller.SetFrameEvent(self.find_action("Stride 0").next.eventId, False)
 
         self.cfg.position = self.controller.GetPostVSData(0, 0, self.cfg.type)
         self.cfg.position.nearPlane = 1.0
@@ -387,3 +401,16 @@ class Mesh_Zoo():
         self.check_vertex(105, 65, (rd.ReplayOutput.NoResult, rd.ReplayOutput.NoResult))
         self.check_vertex(115, 135, (rd.ReplayOutput.NoResult, rd.ReplayOutput.NoResult))
 
+        self.controller.SetFrameEvent(self.find_action("Empty").next.eventId, False)
+
+        self.cfg.position = self.controller.GetPostVSData(0, 0, self.cfg.type)
+        self.cfg.position.nearPlane = 1.0
+        self.cfg.position.farPlane = 100.0
+
+        self.cache_output()
+
+        self.check_vertex(105, 65, (rd.ReplayOutput.NoResult, rd.ReplayOutput.NoResult))
+        self.check_vertex(115, 135, (rd.ReplayOutput.NoResult, rd.ReplayOutput.NoResult))
+
+        rdtest.log.success("Picking in empty draw is as expected")
+
