@@ -49,11 +49,18 @@ class VK_Simple_Triangle(rdtest.TestCase):
 
         self.check_mesh_data(postvs_ref, postvs_data)
 
+        save_data = rd.TextureSave()
+        save_data.destType = rd.FileType.DDS
+        path = rdtest.get_tmp_path('temp.dds')
+
         # Check that nothing breaks if we call typical enumeration functions on resources
         for res in self.controller.GetResources():
             res: rd.ResourceDescription
 
+            save_data.resourceId = res.resourceId
+
             self.controller.GetShaderEntryPoints(res.resourceId)
             self.controller.GetUsage(res.resourceId)
             self.controller.GetBufferData(res.resourceId, 0, 0)
             self.controller.GetTextureData(res.resourceId, rd.Subresource())
+            self.controller.SaveTexture(save_data, path)
